/*
	File:		KeyboardDiagram.c

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<19>	11/14/95	dvb		Conditionally, use public interfaces.
		<18>	 9/13/95	JB		music dumpfile for new build
		<17>	  8/2/95	dvb		shift key
		<16>	 7/31/95	dvb		shift key and control key
		<15+> shift-key to sustain notes (thx jimn8!)
		<15>	 7/26/95	JB		use dump file
		<14>	 7/12/95	dvb		Added track-key routine
		<13+> include keyboard tracking routine
		<13>	 6/14/95	JB		Get rid of phantom black key on right side of keyboard
		<12>	 5/23/95	dvb		lock pictresource before using it
		<11>	  3-5-95	dvb		error updating vector. prototypes.
		<10>	 24-3-95	dvb		name collision
		 <9+>	gogray->kdgogray
		 <9>	 17-3-95	dvb		include the final high-C note of the last octave.
		 <8>	 15-3-95	dvb		Quit bus erroring on drawkeyboardpiece
		 <7+>dont buserr on resload failure	
		 <7>	 30-8-94	dvb		
		 <7>	 30-8-94	dvb		
		 <6>	 24-8-94	dvb		linked build
		 <5>	 4/13/94	dvb		GoGray keeps pnmode
		 <4>	  3/9/94	dvb		GetKeyboardKey offset by low key.
		 <3>	 2/17/94	dvb		resource numbers
		 <2>	 2/14/94	dvb		Velocity Vector
		 <1>	  2/4/94	dvb		first checked in

*/


#ifdef MUSICDUMP
	#ifdef DUMP_USEINCLUDE
		#include MUSICDUMP
	#else
		#pragma load MUSICDUMP
	#endif	
#else
	#include "QTEnv.h"

	#include <Memory.h>
	#include <QuickDraw.h>
	#include <Resources.h>
#endif

#include "KeyboardDiagram.h"

static Handle GetAndDetachResource(ResType type,short id);
static void MaybeGrayRect(KeyboardDiagram *kd,Rect *r);


#define kBlackKeyBottom 16

static Handle GetAndDetachResource(ResType type,short id)
	{
	Handle h;

	h = GetResource(type,id);
	DetachResource(h);
	HNoPurge(h);
	return h;
	}

void InitializeKeyboard(KeyboardDiagram *kd,Rect *r,short lowKey,
		short wkRes,short bkRes,short pict1Res,short pict2Res)
	{
	short i;

	kd->wkRes = wkRes;
	kd->bkRes = bkRes;
	kd->pict1Res = pict1Res;
	kd->pict2Res = pict2Res;

	kd->r = *r;
	kd->keyCount = (r->right - r->left) / kKeyWidth;
	kd->octaveCount = kd->keyCount / 7;
	kd->hiC = (kd->keyCount % 7) == 1;

	kd->keyCount = kd->octaveCount * 12 + kd->hiC;

	if(lowKey >= 128)
		{
		kd->mod = true;
		kd->modOffset = ((lowKey + kd->keyCount - 1) / kd->keyCount) * kd->keyCount;
		lowKey &= 127;
		}
	else
		kd->mod = false;

	kd->lowKey = lowKey;

	for(i = 0; i < 128; i++)
		kd->keyVel[i] = 0;

	kd->whiteKey = (short **)GetAndDetachResource('Junk',kd->wkRes);
	kd->keyRight = (short **)GetAndDetachResource('Junk',kd->bkRes);

	kd->lastKey = 0;
	}

void TerminateKeyboard(KeyboardDiagram *kd)
	{
	DisposeHandle((Handle)kd->whiteKey);
	DisposeHandle((Handle)kd->keyRight);
	}

void SetKeyboardTopLeft(KeyboardDiagram *kd,short top, short left)
	{
	OffsetRect(&kd->r, left - kd->r.left, top - kd->r.top);
	}

void MaybeGrayRect(KeyboardDiagram *kd,Rect *r)
/*
 * modifies the parameter R
 */
	{
	if(kd->gray)
		{
		kdGoGray();
		PenMode(patBic);
		r->right += 2;
		r->bottom += 2;
		PaintRect(r);
		PenNormal();
		}
	}


void DrawKeyboardPiece(KeyboardDiagram *kd, short octave)
	{
	Rect r;
	short w;
	PicHandle pH;

	PenNormal();

	pH = (PicHandle)GetResource('PICT',kd->pict1Res);
	if(pH)
		{
		HNoPurge((Handle)pH);
		r = (**pH).picFrame;
		w = r.right - r.left-1;
		OffsetRect(&r,kd->r.left - r.left + w * octave,kd->r.top - r.top);
		DrawPicture(pH,&r);
		MaybeGrayRect(kd,&r);
		}
	}
	

void DrawKeyboardDropShadow(KeyboardDiagram *kd)
	{
	MoveTo(kd->r.left + 3,kd->r.bottom);
	LineTo(kd->r.right,kd->r.bottom);
	LineTo(kd->r.right,kd->r.top + 3);
	}

void DrawKeyboard(KeyboardDiagram *kd)
	{
	short i;

	DrawKeyboardDropShadow(kd);
	for(i = 0; i < kd->octaveCount; i++)
		DrawKeyboardPiece(kd,i);

	if(kd->hiC)
		{
		PicHandle pH;
		Rect r;

		pH = (PicHandle)GetResource('PICT',kd->pict2Res);
		if(pH)
			{
			r = (**pH).picFrame;
			OffsetRect(&r,kd->r.left - r.left + kd->octaveCount * kKeyWidth * 7,kd->r.top - r.top);
			DrawPicture(pH,&r);
			MaybeGrayRect(kd,&r);
			}
		}

		{
		unsigned char *w;
		w = kd->keyVel;
		for(i = 0; i < 128; i++)
			*w++ = 0;
		}
	}

void SetKeyboardGray(KeyboardDiagram *kd,Boolean gray)
	{
	if(kd->gray != gray)
		{
		kd->gray = gray;
		DrawKeyboard(kd);
		}
	}



void PaintKeyboardKey(KeyboardDiagram *kd,short pitch,short gray)
/* 
 * 0-255, gray; -1, invert; -2, forecolor
 */
	{
	short i,k;
	short octave;
	Rect r1,r2;
	Boolean blackKey;
	short startPitch;

	if(kd->gray)
		goto goHome;


	r1.left = r1.right = 0;
	r2.left = r2.right = 0;

	pitch -= kd->lowKey;
	if(pitch < 0 || pitch >= kd->keyCount)
		goto goHome;
	
	startPitch = pitch;
	octave = pitch / 12;
	pitch = pitch % 12;

	blackKey = false;

	if(pitch == 1 || pitch == 3
			|| pitch == 6 || pitch == 8 || pitch == 10)
		{
		/*
		 * Black Keys
		 */
		blackKey = true;
		r1.left = kd->r.left + (*kd->keyRight)[pitch - 1] + 2;
		r1.top = kd->r.top + 3;
		r1.right = kd->r.left + (*kd->keyRight)[pitch];
		r1.bottom = kd->r.top + kBlackKeyBottom;
		}
	else
		{
		/*
		 * White Keys
		 */
		k = kd->r.left + 2;
		i = 0;
		while((*kd->whiteKey)[i] != pitch)
			{
			i++;
			k += 9;
			}
		r1.left = k;
		r1.top = kd->r.top + kBlackKeyBottom + 2;
		r1.right = k+6;
		r1.bottom = kd->r.bottom - 1;

		if ((startPitch == kd->keyCount-1) && (pitch == 0)) {
			r2.top = kd->r.top + 2;
			r2.bottom = kd->r.top + kBlackKeyBottom + 2;
			r2.left = k;
			r2.right = k+6;
		} else {
			if(pitch)
				r2.left = kd->r.left + (*kd->keyRight)[pitch - 1] + 2;
			else
				r2.left = kd->r.left + 2;
			r2.top = kd->r.top + 2;
			r2.right = kd->r.left + (*kd->keyRight)[pitch];
			r2.bottom = kd->r.top + kBlackKeyBottom + 2;
		}

		/*
		 * This is cheesy and proves I have
		 * no idea what I am doing. - dvb
		 */
		if(pitch == 5)
			r2.left ++;
		else if(pitch == 11)
			r2.right --;
		}

	OffsetRect(&r1,octave * 63,0);
	OffsetRect(&r2,octave * 63,0);

	if(gray >=0)
		{
		RGBColor c;

		if(!blackKey)
			gray = 255 - gray;
		c.blue = c.green = c.red = (((unsigned short)gray)<<8) | gray;
		RGBForeColor(&c);
		}

	if(gray == -1)
		{
		InvertRect(&r1);
		InvertRect(&r2);
		}
	else
		{
		PaintRect(&r1);
		PaintRect(&r2);
		}
goHome:
	ForeColor(blackColor);
	PenNormal();
	}

void kdGoGray(void)
	{
	Pattern p;
	long *w;

	w = (long *)&p;
	*w++ = 0xAA55AA55;
	*w++ = 0xAA55AA55;
	PenPat(&p);
	}

void PaintKeyboardVector(KeyboardDiagram *kd,unsigned char *keyVel)
	{
	short i;
	unsigned char *w1,*w2;
	long x;

	w1 = keyVel;
	w2 = kd->keyVel;
	for(i = 0; i < 128; i++)
		{
		x = *w1;
		if(*w2 != x)
			{
			*w2 = x;
			x = x ? x+128:0;
			if( (i < kd->lowKey) || (i >= (kd->lowKey + kd->keyCount)) )
				{
				if(kd->mod)
					PaintKeyboardKey(kd,
							(i+kd->modOffset)%(kd->keyCount)+kd->lowKey,x);
				}
			PaintKeyboardKey(kd,i,x);
			}
		w1++;
		w2++;
		}
	}


short GetKeyboardKey(KeyboardDiagram *kd,Point p)
/*
 * Return -1 if outside rectangle.
 */
	{
	short pitch,octave;
	Rect r;

	r = kd->r;
	r.right--;

	if(PtInRect(p,&r))
		{
		p.h -= kd->r.left;
		p.v -= kd->r.top;

		octave = p.h / 63;
		p.h %= 63;

		if(p.v > kBlackKeyBottom)
			pitch = (*kd->whiteKey)[p.h/9];
		else
			{
			pitch = 0;
			while((*kd->keyRight)[pitch] < p.h)
				pitch++;
			}
		pitch += octave*12;
		if (pitch >= kd->keyCount)
			pitch = kd->keyCount - 1;
		pitch += kd->lowKey;
		}
	else
		pitch = -1;

	return pitch;
	}

short ClickKeyboard(KeyboardDiagram *kd,Point p,short mods,NoteAllocator na,NoteChannel nc)
	{
	short pitch,vel;

	pitch = GetKeyboardKey(kd,p);
	if(pitch < 0)
		pitch = 0;

	if(nc)
		{			//  send sustain pedal for shift key
		short mh;

		mh = mods & shiftKey;
		if(mh != (kd->lastMods & shiftKey))
			NASetController(na,nc,kControllerSustain,mh ? 0x7FFF : 0);

		vel = 64 + ((mods & cmdKey) ? 35 : 0)
				+ ((mods & optionKey) ? 20 : 0);
		}


	if(mods & controlKey)
		{
		if( pitch && ((!(kd->lastMods & controlKey)) || (!kd->lastKey)))	//  start bending if we have a pitch and either the control key was just pressed, or we just hit a pitch
			{
			kd->basePoint = p;
			}

		else if( (kd->lastMods & controlKey) && kd->basePoint.h && p.h) //  already bending if still control key AND we started bending
			{
			pitch = kd->lastKey;		//  inhibit pitch change
			if((p.h != kd->lastP.h) && nc)
				NASetController(na,nc,kControllerPitchBend,
						(p.h - kd->basePoint.h) * ((long)0x0c00) / (7 * kKeyWidth) );
			}
		}

	//  Draw and play the difference between pitch and lastKey
	if(pitch != kd->lastKey)
		{
		PaintKeyboardKey(kd,pitch,-1);
		PaintKeyboardKey(kd,kd->lastKey,-1);

		//  Stop old note
		if(kd->lastKey && nc)
			{
			NAPlayNote(na,nc,kd->lastKey,0);
			NASetController(na,nc,kControllerPitchBend,0);
			}

		//  Play new note
		if(pitch && nc)
			NAPlayNote(na,nc,pitch,vel);
		}

	kd->lastMods = mods;
	kd->lastKey = pitch;
	kd->lastP = p;
	return pitch;
	}


