/*
	File:		NAEditInstSlider.c

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	

	Change History (most recent first):

		<23>	11/14/95	dvb		Conditionally, use public interfaces.
		<22>	 9/13/95	JB		music dumpfile for new build
		<21>	 9/12/95	mdd		include Fonts.h so that we build with wayCoolBuild
		<20>	  9/6/95	dvb		allow nil for settings
		<19>	 8/30/95	dvb		hier popups
		<18>	 8/24/95	dvb		online helpp
		<17>	 8/22/95	dvb		inst controls may have categorized menus
		<16>	 8/21/95	JLN		Display negatives correctly in 8.8 fixed slider
		<15>	 8/15/95	JLN		Read-only knob support
		<14>	 8/11/95	JLN		Support Note knob type (8.8 Fixed)
		<13>	  8/9/95	JLN		User interface for pan controller slider
		<12>	 7/26/95	JB		use dump file
		<10>	 6/28/95	dvb		let quick clicks in thumb set value
		 <9>	 6/14/95	JB		prototypes and unused vars
		 <8>	 4/12/95	JB		all -d must be defined as either TRUE or FALSE. no more #ifdefs.
									use #ifs instead.
		 <7>	  6-4-95	dvb		show '?' in popups of unknown value
		 <6>	  4/3/95	GMA		ifdef's some procs under wayCoolBuild because they were multiply
									defined.
		 <5>	 24-3-95	dvb		misc -- menus must be inserted beforehand
		 <4>	 20-3-95	dvb		save and restore font
		 <3>	 10-2-95	dvb		reformatted
		 <2+>	 reformat slider to horizontal layout
		 <2>	  6-2-95	dvb		misc

*/

#include <Balloons.h>
#include <QDOffscreen.h>
#include <Windows.h>

#ifdef MUSICDUMP
	#ifdef DUMP_USEINCLUDE
		#include MUSICDUMP
	#else
		#pragma load MUSICDUMP
	#endif	
#else
	#include "QTEnv.h"
	#include <Memory.h>
	#include <Fonts.h>
	#include <QDOffscreen.h>

	#ifdef UsePublicMusicInterfaces
		#include <QuickTimeComponents.h>
	#else
		#include "MusicComponent.h"
	#endif
#endif


#include "NAEditInstSlider.h"



/*----------------------------
	Secret Data Structure
----------------------------*/

enum
	{
	kQKBar = 3,
	kQKPopup,
	kQKBoolean,
	kQKGroupName,

	kQKInteger,
	kQKPercentage,
	kQKFixed8,
	kQKFixed16,
	kQKPanRange
	};

typedef struct
	{
	Boolean used;
	QTMAKnobList knobList;
	long knobIndex;
	KnobDescription kd;
	Rect r;
	long value;
	long knobNumber;
	long refCon;
	Boolean active;			/* if not active, dont draw or click it */

	short mainType;			/* from enum above */
	short numberType;		/* for bar number */

// Only for booleans
	Handle boolTags;		/* list of strings for boolean checks */

// Only for sliders
 	short thumbWidth;
 	Rect thumbRect;
	short barLeft,barRight;

// Only for popups
	MCPopup mcp;			/* used if the maintype is kQKPopup */
	} QTMAKnobRecord;


typedef struct
	{
	GWorldPtr gw;
	Rect bufferR;
	GWorldPtr oldGW;
	GDHandle oldGD;

	MusicComponent mc;

	short widest,highest;

	long menuIDSeed;

	long knobCount;
	QTMAKnobRecord qk[1];
	} QTMAKnobListRecord;

typedef struct
	{
	short font,size,face;
	} Fontystuff;

/*--------------------
	Local Prototypes
--------------------*/
static long MapRange(long x,long lowFrom,long highFrom,long lowTo,long highTo);
static long PinRange(long x,long low,long high);
static void GoBW();
static void GetFontystuff(Fontystuff *fs);
static void SetFontystuff(Fontystuff *fs);

static QTMAKnobRecord *GetKnobRecord(QTMAKnobList knobList,long knobIndex);
static void DrawKnobFrameAndText(QTMAKnobRecord *qk);
static void DrawKnobBar(QTMAKnobRecord *qk);
static void DrawKnobThumb(QTMAKnobRecord *qk);
static void DrawKnobBooleans(QTMAKnobRecord *qk);
static void DrawKnobPopup(QTMAKnobRecord *qk);

static void KnobValueToPString(QTMAKnobRecord *qk,long knob_value,Str255 s);
static short FindThumbWidth(QTMAKnobRecord *qk);
static void TrackQTMABar(QTMAKnobList knobList,long knobIndex,
		Point p,short mods,ControlActionUPP actionProc);

static Boolean BeginBuffering(QTMAKnobList knobList,Rect *r);
static void EndBuffering(QTMAKnobList knobList);
static void StringFromStringListH(Handle h, unsigned short index,Str255 outString);


void DrawBooleanCheckbox(Rect *r,short style,long value);
void TrackQTMABoolean(QTMAKnobList knobList,long knobIndex,
		Point p,short mods,ControlActionUPP actionProc);
long GetKnobBitCount(QTMAKnobRecord *qk);
void GetBooleanPosition(QTMAKnobRecord *qk,short bit,short *textRight,Rect *checkR);

static void DrawNum(long n);
static void DrawStringCenter(StringPtr s);
static void DrawStringRight(StringPtr s);
static void TruncateString(StringPtr s, short width);
static void DrawStringTruncated(StringPtr s, short width);
static void CopyPString(StringPtr dest,StringPtr src);
static void AppendPString(StringPtr dest,StringPtr src);

long xmax(long a,long b);

void EnsureBugger(QTMAKnobListRecord *qlr);

void GoGrey(unsigned short g);


/*--------------------
	Format Constants
--------------------*/
#define kQKBarInsetMargin 6
#define kQKThumbInsetMargin 3

#define kQKFontSize 9
#define kQKGroupFontSize 12
#define kQKNameDescent 12
#define kQKThumbHeight 13
#define kQKBarHeight 7
#define kQKThumbTextAscent 3

#define kQKCheckBoxMargin 3
#define kQKCheckBoxSize 12


/*--------------------
	Utility Functions
--------------------*/


long MapRange(long x,long lowFrom,long highFrom,long lowTo,long highTo)
	{
	long a;
	long toRange,fromRange;

	toRange = highTo - lowTo;
	fromRange = highFrom - lowFrom;

	x -= lowFrom;
	a = x;
	a *= toRange;
	a += fromRange >> 1;
	a /= fromRange;
	a += lowTo;
	x = a;

	x = PinRange(x,lowTo,highTo);
	return x;
	}
	

long PinRange(long x,long low,long high)
	{
	if(x < low)
		x = low;
	else if(x > high)
		x = high;

	return x;
	}

void GoBW(void)
	{
	PenNormal();
	ForeColor(blackColor);
	BackColor(whiteColor);
	TextSize(kQKFontSize);
	TextFont(1);
	TextFace(0);
	}


void GetFontystuff(Fontystuff *fs)
	{
	GrafPtr p;

	GetPort(&p);

	fs->face = p->txFace;
	fs->font = p->txFont;
	fs->size = p->txSize;
	}

void SetFontystuff(Fontystuff *fs)
	{
	TextFace(fs->face);
	TextFont(fs->font);
	TextSize(fs->size);
	}



/*--------------------
	The Stuff
--------------------*/


QTMAKnobList NewQTMAKnobList(long knobCount,MusicComponent mc)
	{
	Handle h;
	QTMAKnobListRecord *qlr;

	h = NewHandleClear(sizeof(QTMAKnobListRecord) + (knobCount - 1)*sizeof(QTMAKnobRecord));
	if(h)
		{
		HLock(h);
		qlr = *(QTMAKnobListRecord **)h;
		qlr->knobCount = knobCount;

		qlr->mc = mc;

		qlr->menuIDSeed = 56789;
		}

	return (QTMAKnobList)h;	
	}

void DisposeQTMAKnobListAt(QTMAKnobList *knobList)
	{
	Handle *hh;
	QTMAKnobListRecord *qlr;
	long i;

	hh = (Handle *)knobList;
	if(hh && *hh)
		{
		qlr = **(QTMAKnobListRecord ***)hh;

		for(i = 0; i < qlr->knobCount; i++)
			DisposeQTMAKnob(*knobList,i);

		if(qlr->gw)
			DisposeGWorld(qlr->gw);
		DisposeHandle(*hh);
		*hh = 0;
		}
	}

void DisposeQTMAKnob(QTMAKnobList knobList,long knobIndex)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	if(qk->used)
		{
		if(qk->mainType == kQKPopup)
			TerminateMCPopup(&qk->mcp);

		if(qk->boolTags)
			{
			DisposeHandle(qk->boolTags);
			qk->boolTags = 0;
			}
		qk->used = false;
		}
	}

void EnsureBugger(QTMAKnobListRecord *qlr)
	{
	Rect r;
	OSErr thisError;

	if(!qlr->gw)
		{
		r.top = r.left = 0;
		r.bottom = qlr->highest;
		r.right = qlr->widest;
		thisError = NewGWorld(&qlr->gw, 8, &r, nil, nil, 0);
		LockPixels(qlr->gw->portPixMap);
		}
	}

Boolean BeginBuffering(QTMAKnobList knobList,Rect *r)
	{
	QTMAKnobListRecord *qlr;
	Boolean visible;

	qlr = *(QTMAKnobListRecord **)knobList;

	GetGWorld(&qlr->oldGW,&qlr->oldGD);
	visible = RectInRgn(r,qlr->oldGW->visRgn)
			&& RectInRgn(r,qlr->oldGW->clipRgn);

	if(visible)
		{
		EnsureBugger(qlr);
		qlr->bufferR = *r;
	
	
		SetGWorld(qlr->gw,nil);
		SetOrigin(r->left,r->top);
		}

	return visible;
	}

void EndBuffering(QTMAKnobList knobList)
	{
	PixMapHandle pmH;
	QTMAKnobListRecord *qlr;

	qlr = *(QTMAKnobListRecord **)knobList;

	pmH = qlr->gw->portPixMap;

	SetGWorld(qlr->oldGW,qlr->oldGD);
	CopyBits((BitMap *)*pmH,(BitMap *)&qlr->oldGW->portPixMap,
			&qlr->bufferR,&qlr->bufferR,
			0,0);
	}


extern void MyNumToString(long n,short g,StringPtr c);

void NewQTMAKnob(QTMAKnobList knobList,long knobIndex,
		KnobDescription *kd,Rect *r,long knobNumber,long refCon)
	{
	QTMAKnobRecord *qk;
	QTMAKnobListRecord *qlr;
	Fontystuff fs;

	GetFontystuff(&fs);

	GoBW();

	qlr = *(QTMAKnobListRecord **)knobList;
	qk = GetKnobRecord(knobList,knobIndex);

	qk->used = true;
	qk->active = true;

	qk->knobList = knobList;
	qk->knobIndex = knobIndex;

		{
		short x;

		x = r->right - r->left;
		if(x > qlr->widest)
			qlr->widest = x;

		x = r->bottom - r->top;
		if(x > qlr->highest)
			qlr->highest = x;
		}

	if(kd)
		qk->kd = *kd;
	else
		{
		qk->kd.name[0] = 0;
		qk->kd.lowValue = 0;
		qk->kd.highValue = 127;
		qk->kd.defaultValue = 1;
		qk->kd.flags = 0;
		}
	qk->r = *r;
	qk->knobNumber = knobNumber;
	qk->refCon = refCon;
	qk->value = qk->kd.defaultValue;

	qk->barLeft = (qk->r.left + qk->r.right + 1)>>1;
	qk->barRight = qk->r.right - kQKThumbInsetMargin;

	switch(qk->kd.flags & 0xF000)
		{
		case kKnobTypeGroupName:
			qk->mainType = kQKGroupName;
			break;

		case kKnobTypeBoolean:
			qk->mainType = kQKBoolean;
			break;

		case kKnobTypeNote:
			qk->mainType = kQKBar;
			qk->numberType = kQKFixed8;
			break;

		case kKnobTypePan:
			qk->mainType = kQKBar;
			qk->numberType = kQKPanRange;
			break;
#define kMenuMargin (kMCPopupBoxMargin+5)
#define kMenuHeight 7

		case kKnobTypeSetting:
				{
				Rect r2;
				short i;

				qk->mainType = kQKPopup;
				r2.left = qk->barLeft + kMenuMargin;
				r2.right = qk->barRight - kMenuMargin;
				r2.bottom = r->bottom - kMenuMargin + 2;
				r2.top = r2.bottom - kMenuHeight;
				InitializeMCPopup(&qk->mcp,&r2,nil,qlr->menuIDSeed++,0);
				for(i = qk->kd.lowValue; i <= qk->kd.highValue; i++)
					{
					Str255 s;

					MyNumToString(i+1-qk->kd.lowValue,0,s);
					AddMCPopupItem(&qk->mcp,s,i);
					}
				}
			break;

		case kKnobTypePercentage:
			qk->mainType = kQKBar;
			qk->numberType = kQKPercentage;
			break;

		default:
			qk->mainType = kQKBar;
			qk->numberType = kQKInteger;
			if(qk->kd.highValue >=65535)
				qk->numberType = kQKPercentage;

			break;
		}

	qk->thumbWidth = FindThumbWidth(qk);

	SetFontystuff(&fs);
	}

void DrawQTMAKnob(QTMAKnobList knobList,long knobIndex)
	{
	QTMAKnobRecord *qk;
	Boolean visible;
	Fontystuff fs;

	GetFontystuff(&fs);

	qk = GetKnobRecord(knobList,knobIndex);

	if(!(qk->used && qk->active))
		goto goHome;

	visible = BeginBuffering(knobList,&qk->r);
	if(!visible)
		goto goHome;

	DrawKnobFrameAndText(qk);

	switch(qk->mainType)
		{
		case kQKBar:
			DrawKnobBar(qk);
			DrawKnobThumb(qk);
			break;

		case kQKGroupName:
			break;

		case kQKBoolean:
			DrawKnobBooleans(qk);
			break;

		case kQKPopup:
			DrawKnobPopup(qk);
			break;

		default:
			MoveTo(qk->r.left + 10,qk->r.bottom - 4);
			DrawNum(qk->value);
			break;
		}

	EndBuffering(knobList);
goHome:;
	SetFontystuff(&fs);
	}

void DrawKnobPopup(QTMAKnobRecord *qk)
	{
	SetMCPopupChoice(&qk->mcp,qk->value);

	if (qk->kd.flags & kKnobReadOnly)
		{
		Str255 s;
Debugger();
		GetMCPopupString(&qk->mcp,s);
		MoveTo(qk->barLeft + 10,qk->r.bottom - 6);
		DrawString(s);
		}
	else
		DrawMCPopup(&qk->mcp);
	}

Boolean HitQTMAKnob(QTMAKnobList knobList,long knobIndex,Point p)
	{
	Boolean result;
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);
	if(!(qk->used && qk->active))
		result = false;
	else
		result = PtInRect(p,&qk->r); // && (qk->mainType != kQKGroupName);

	return result;
	}

#define kSlop 5

void TrackQTMAKnob(QTMAKnobList knobList,long knobIndex,Point p,short mods,ControlActionUPP actionProc)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	if(p.h < qk->barLeft)		/* in the text area? */
		{
		OSErr result;
		HMMessageRecord mr;
		Boolean wasBalloon;
		Point q;
		Rect r;

		q = p;
	
		wasBalloon = HMGetBalloons();
		result = HMSetBalloons(true);
	
		mr.hmmHelpType = khmmString;
		BlockMove("\pHelp!",mr.u.hmmString,6);
	
		LocalToGlobal(&q);
		r = qk->r;
		LocalToGlobal((Point *)&r.top);
		LocalToGlobal((Point *)&r.bottom);
		result = HMShowBalloon(&mr, q, &r, nil, 0, 1, kHMRegularWindow);
	
		while(StillDown())
			;
	
		result = HMSetBalloons(wasBalloon);
		goto goHome;
		}

	if (qk->kd.flags & kKnobReadOnly)
	{
		SysBeep(1);
		goto goHome;		/* don't allow editing of read-only knobs */
	}
// switch to different kinds of tracking

	if(mods & cmdKey)
		{
		SetQTMAKnobValue(knobList,knobIndex,qk->kd.defaultValue);
		if(actionProc)
			(actionProc)((ControlRef)qk,1);
		}
	else if(qk->mainType == kQKBoolean)
		TrackQTMABoolean(knobList,knobIndex,p,mods,actionProc);
	else if(qk->mainType == kQKPopup)
		{
		TrackMCPopup(&qk->mcp,p);
		qk->value = GetMCPopupChoice(&qk->mcp);
		if(actionProc)
			(actionProc)((ControlRef)qk,1);
		}
	else
		TrackQTMABar(knobList,knobIndex,p,mods,actionProc);

goHome:;
	}



void TrackQTMABoolean(QTMAKnobList knobList,long knobIndex,
		Point p,short mods,ControlActionUPP actionProc)
	{
	short bitCount,i;
	Rect r,slopR;
	long shiftI,v1;
	Boolean in,lastIn;
	long oldValue,clickValue,newValue,partCode;

	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	if(qk->value == kUnknownKnobValue)
		qk->value = qk->kd.defaultValue;

	bitCount = GetKnobBitCount(qk);
	for(i = 0; i < bitCount; i++)
		{
		GetBooleanPosition(qk,i,&slopR.right,&r);
		slopR.top = r.top;
		slopR.bottom = r.bottom;
		slopR.left = r.left;
		if(PtInRect(p,&slopR))
			{
			shiftI = (1<<i);
			lastIn = false;
			in = true;
			v1 = qk->value & shiftI;
			oldValue = qk->value;
			clickValue = qk->value ^ shiftI;
			do
				{
				in = PtInRect(p,&slopR);
				if(in != lastIn)
					DrawBooleanCheckbox(&r,
							PtInRect(p,&slopR) ? 1 : 0,v1);

				if(in)
					newValue = clickValue;
				else
					newValue = oldValue;
	
				partCode = newValue != qk->value;
	
				SetQTMAKnobValue(knobList,knobIndex,newValue);
	
				if(actionProc)
					(actionProc)((ControlRef)qk,partCode);

				lastIn = in;
				GetMouse(&p);
				} while(StillDown());

			if(in)
				qk->value ^= shiftI;

			SetQTMAKnobValue(knobList,knobIndex,newValue);

			if(actionProc)
				(actionProc)((ControlRef)qk,1);

			goto goHome;
			}
		}

goHome:;
	}



void TrackQTMABar(QTMAKnobList knobList,long knobIndex,
		Point p,short mods,ControlActionUPP actionProc)
	{
	long newValue,oldValue;
	Rect slopR;
	short partCode;
	short hShift;
	short left,right;
	short originalH;
	QTMAKnobRecord *qk;
	Boolean first;

	qk = GetKnobRecord(knobList,knobIndex);

	oldValue = qk->value;
	if(oldValue == kUnknownKnobValue)
		oldValue = qk->kd.defaultValue;

	slopR = qk->r;
	InsetRect(&slopR,-kSlop,-kSlop);

	left = qk->barLeft;
	right = qk->barRight - qk->thumbWidth;

	if(PtInRect(p,&qk->thumbRect))
		{
		Point p2;

		originalH = p.h;		/* so if it's a subpixel value, the value wont move */
		hShift = p.h - qk->thumbRect.left;
		if(mods & controlKey)
			{
			p2 = p;
			do
				{
				GetMouse(&p2);
				newValue = oldValue + p2.h - p.h;
	
				partCode = (newValue != qk->value) ? 1 : 0;
				SetQTMAKnobValue(knobList,knobIndex,newValue);
				if(actionProc)
					(actionProc)((ControlRef)qk,partCode);
				} while(StillDown());
			goto goHome;
			}
		}


	else
		{
		originalH = slopR.left - 20;		/* if bar was clicked, there's no return point */
		hShift = qk->thumbWidth / 2;
		}


	first = true;		/* so a quick click on the thumb will still resend the value */
	do
		{
		if(p.h == originalH || !PtInRect(p,&slopR))
			newValue = oldValue;
		else
			newValue = MapRange(p.h - hShift,left,right,qk->kd.lowValue,qk->kd.highValue);
	
		partCode = ((newValue != qk->value) || first) ? 1 : 0;
		SetQTMAKnobValue(knobList,knobIndex,newValue);

		if(actionProc)
			(actionProc)((ControlRef)qk,partCode);

		GetMouse(&p);
		first = false;
		} while(StillDown());
goHome:;
	}






void SetQTMAKnobValue(QTMAKnobList knobList,long knobIndex,long value)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	if(qk->value != value)
		{
		if(value != kUnknownKnobValue)
			value = PinRange(value,qk->kd.lowValue,qk->kd.highValue);
		qk->value = value;
		DrawQTMAKnob(knobList,knobIndex);
		}
	}


long GetQTMAKnobValue(QTMAKnobList knobList,long knobIndex)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	return qk->value;
	}

long GetQTMAKnobNumber(QTMAKnobList knobList,long knobIndex)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	return qk->knobNumber;
	}

long GetQTMAKnobRefCon(QTMAKnobList knobList,long knobIndex)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	return qk->refCon;
	}


void GetQTMAKnobRect(QTMAKnobList knobList, long knobIndex, Rect *r)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	*r = qk->r;
	}


void GoGrey(unsigned short g)
	{
	RGBColor rgb;

	rgb.red = rgb.green = rgb.blue = g;
	RGBForeColor(&rgb);
	}


#ifdef oldStackedStyle
void DrawKnobFrameAndText(QTMAKnobRecord *qk)
	{
	Rect r;

	GoBW();
	GoGrey(32768);
	FrameRect(&qk->r);
	r = qk->r;
	GoGrey(56000);
	InsetRect(&r,1,1);
	PaintRect(&r);
	GoBW();

	if(qk->mainType == kQKGroupName)
		{
		TextSize(kQKGroupFontSize);
		TextFace(bold);
		MoveTo((r.left + r.right + 1)>>1,(r.top + r.bottom + kQKGroupFontSize)>>1);
		DrawStringCenter(qk->kd.name);
		goto goHome;
		}

	MoveTo(r.left + kQKBarInsetMargin,r.top + kQKNameDescent);
	DrawString(qk->kd.name);
	MoveTo(r.right - kQKBarInsetMargin,r.top + kQKNameDescent);
	switch(qk->kd.flags & 0xF000)
		{
		case kKnobTypeMilliseconds:
			DrawStringRight("\pmS");
			break;
		case kKnobTypePercentage:
			DrawStringRight("\p%");
			break;
		case kKnobTypeHertz:
			DrawStringRight("\pHz");
			break;
		}	

goHome:;
	}


#else
void DrawKnobFrameAndText(QTMAKnobRecord *qk)
	{
	Rect r;
	short textRight;		/* position for units tag, if any */

	r = qk->r;
	InsetRect(&r,1,1);
	GoBW();
	GoGrey(22000);
	MoveTo(qk->r.left,r.bottom);
	LineTo(qk->r.left,qk->r.top);
	LineTo(r.right, qk->r.top);
	GoGrey(56000);
	Move(0,1);		/* down one to miss the corners */
	LineTo(r.right,r.bottom);
	LineTo(r.left,r.bottom);
	GoBW();
	EraseRect(&r);

	if(qk->mainType == kQKGroupName)
		{
		TextSize(kQKGroupFontSize);
		TextFace(bold);
		MoveTo((r.left + r.right + 1)>>1,(r.top + r.bottom + kQKGroupFontSize)>>1);
		DrawStringCenter(qk->kd.name);
		goto goHome;
		}

	MoveTo(r.left + kQKBarInsetMargin,r.top + kQKNameDescent);
	DrawString(qk->kd.name);
	textRight = ((r.left + r.right)>>1) - kQKBarInsetMargin;
	MoveTo(textRight,r.top + kQKNameDescent);
	switch(qk->kd.flags & 0xF000)
		{
		case kKnobTypeMilliseconds:
			DrawStringRight("\pmS");
			break;
		case kKnobTypePercentage:
			DrawStringRight("\p%");
			break;
		case kKnobTypeHertz:
			DrawStringRight("\pHz");
			break;
		}	

goHome:;
	}


#endif


void DrawKnobBar(QTMAKnobRecord *qk)
	{
	Rect r;
	short left,right;

	if (qk->kd.flags & kKnobReadOnly)
		goto goHome;

	left = qk->barLeft;
	right = qk->barRight;

	r.left = left;
	r.right = right;
	r.bottom = qk->r.bottom - kQKBarInsetMargin;
	r.top = r.bottom - kQKBarHeight;
	FrameRoundRect(&r,(kQKBarHeight+1)/2,(kQKBarHeight+1)/2);

	if(qk->value == kUnknownKnobValue)
		{
		}
	else
		{
		InsetRect(&r,2,2);
		FrameRoundRect(&r,(kQKBarHeight-4),(kQKBarHeight-4));
		}
goHome:;
	}

void DrawKnobThumb(QTMAKnobRecord *qk)
	{
	short left,right;
	Str255 s;
	long v;

	if(qk->value == kUnknownKnobValue)
		{
		s[0] = 1;
		s[1] = '?';		//!!! string list
		}
	else
		KnobValueToPString(qk,qk->value,s);

	left = qk->barLeft;
	right = qk->barRight - qk->thumbWidth;

	qk->thumbRect.bottom = qk->r.bottom - kQKThumbInsetMargin;
	qk->thumbRect.top = qk->thumbRect.bottom - kQKThumbHeight;

	if (qk->kd.flags & kKnobReadOnly)
		{
		MoveTo(left,qk->thumbRect.bottom);
		DrawString(s);
		}
	else
		{
		if(qk->value == kUnknownKnobValue)
			v = qk->kd.defaultValue;
		else
			v = qk->value;

		qk->thumbRect.left = MapRange(v,qk->kd.lowValue,qk->kd.highValue,left,right);
		qk->thumbRect.right = qk->thumbRect.left + qk->thumbWidth;
	
		EraseRect(&qk->thumbRect);
		FrameRect(&qk->thumbRect);
		MoveTo((qk->thumbRect.left + qk->thumbRect.right + 1)>>1,qk->thumbRect.bottom - kQKThumbTextAscent);
		DrawStringCenter(s);
		}
goHome:;
	}


long GetKnobBitCount(QTMAKnobRecord *qk)
	{
	unsigned long x,bitCount;

	x = qk->kd.highValue;
	bitCount = 0;
	while(x)
		{
		x >>= 1;
		bitCount++;
		}

	return bitCount;
	}

void GetBooleanPosition(QTMAKnobRecord *qk,short bit,short *textRight,Rect *checkR)
	{
	Rect r;
	short left,right;
	long bitCount;

	bitCount = GetKnobBitCount(qk);

	left = qk->barLeft;
	right = qk->barRight;
	r.bottom = qk->r.bottom - kQKCheckBoxMargin;
	r.top = r.bottom - kQKCheckBoxSize;

	if(textRight)
		*textRight = MapRange(bit + 1,0,bitCount,left,right);
	r.left = MapRange(bit,0,bitCount,left,right) + kQKCheckBoxMargin;
	r.right = r.left + kQKCheckBoxSize;

	if(checkR)
		*checkR = r;
	}

void DrawBooleanCheckbox(Rect *r,short style,long value)
	{
	short x;
	Rect rr;
	Pattern greyP,blackP;

		{
		register long *w;
		w = (long *)&greyP;
		*w++ = 0xAA55AA55;
		*w++ = 0xAA55AA55;
		w = (long *)&blackP;
		*w++ = 0xFFFFFFFF;
		*w++ = 0xFFFFFFFF;
		}

	x = 1;
	if(style == 1)
		{
		PenSize(2,2);
		x = 2;
		}
	else if(style == 2)
		PenPat(&greyP);

	FrameRect(r);
	PenSize(1,1);
	rr = *r;
	InsetRect(&rr,x,x);
	EraseRect(&rr);

	if(value)
		{
		rr.right -= 1;
		rr.bottom -= 1;

		MoveTo(rr.left,rr.top);
		LineTo(rr.right,rr.bottom);
		MoveTo(rr.left,rr.bottom);
		LineTo(rr.right,rr.top);
		}

	PenPat(&blackP);
	}

void StringFromStringListH(Handle h, unsigned short index,Str255 outString)
/*
 * index is 0..(n-1)
 */
	{
	unsigned char *w;

	if(!h)
		{
		outString[0] = 0;
		}
	else
		{
		w = *(StringPtr *)h;
		if(index >= *(short *)w)
			w += 2;
		else
			{
			w += 2;
			while(index-- > 0)
				w += (*w) + 1;
			}
		BlockMove(w,outString,(*w)+1);
		}
	}



void DrawKnobBooleans(QTMAKnobRecord *qk)
	{
	short bitCount,i;
	short left,right;
	Rect r;
	Str255 s;

	bitCount = GetKnobBitCount(qk);

	for(i = 0; i < bitCount; i++)
		{
		GetBooleanPosition(qk,i,&right,&r);
		DrawBooleanCheckbox(&r,
				qk->value == kUnknownKnobValue ? 2 : 0,(1<<i) & qk->value);

		left = r.right + kQKCheckBoxMargin;
		MoveTo(left,r.bottom - 2);
		StringFromStringListH(qk->boolTags,i,s);
		DrawStringTruncated(s,right - left);
		}
	}




long xmax(long a,long b)
	{
	return a > b ? a : b;
	}

short FindThumbWidth(QTMAKnobRecord *qk)
	{
	Str255 s;
	short w;

	GoBW();
	KnobValueToPString(qk,qk->kd.highValue,s);
	w = StringWidth(s);

	KnobValueToPString(qk,qk->kd.lowValue,s);
	w = xmax(w,StringWidth(s));

	KnobValueToPString(qk,qk->kd.defaultValue,s);
	w = xmax(w,StringWidth(s));

	KnobValueToPString(qk, (qk->kd.lowValue + qk->kd.highValue) >> 1,s);
	w = xmax(w,StringWidth(s));

/* Get max. string width more accurately when dealing with 8.8 fixed */
	if (qk->kd.lowValue + 1 < qk->kd.highValue)
	{	
		KnobValueToPString(qk, qk->kd.highValue - 1,s);
		w = xmax(w,StringWidth(s));
		KnobValueToPString(qk, qk->kd.lowValue + 1,s);
		w = xmax(w,StringWidth(s));
	}

	return w + 10;
	}


void KnobValueToPString(QTMAKnobRecord *qk,long knob_value,Str255 s)
{
	long v;
	short v2;
	Str255 s2;
	switch (qk->numberType)
	{
		case kQKPanRange:
			if ((knob_value >= 0x17C) && (knob_value <= 0x184))
				CopyPString(s, "\pCenter");
			else
			{
				if (knob_value < 0x180)
				{
					v = 0x180 - knob_value;
					CopyPString(s, "\pL ");
				}
				else
				{
					v = knob_value - 0x180;
					CopyPString(s, "\pR ");
				}
	
				v = (v * 100) / 128;
				MyNumToString(v,0,s2);
				AppendPString(s, s2);
				AppendPString(s, "\p%");
			}
			break;

		case kQKFixed8:
			if (knob_value < 0)
			{
				CopyPString(s, "\p-");
				knob_value = -knob_value;
				MyNumToString(knob_value >> 8, 0, s2);
				AppendPString(s, s2);
			}
			else
				MyNumToString(knob_value >> 8, 0, s);

			v2 = knob_value & 0xFF;
			if (v2)
			{
				AppendPString(s, "\p.");
				if (knob_value & 0x8000)
					v2 = 256 - v2;
				v2 = (v2 * 100) >> 8;
				if (v2 > 99)
					v2 = 99;
				if (v2 == 0)
					v2 = 1;		/* show .00 as .01 instead */
				MyNumToString(v2, 0, s2);
				if (v2 < 10)
					AppendPString(s, "\p0");
				AppendPString(s, s2);
			}
			else
				AppendPString(s, "\p.00");
			break;
		case kQKPercentage:
			knob_value = MapRange(knob_value,qk->kd.lowValue,qk->kd.highValue,0,100);
			MyNumToString(knob_value,0,s);
			AppendPString(s, "\p%");
			break;

		default:
			MyNumToString(knob_value,0,s);
	}
}


static QTMAKnobRecord *GetKnobRecord(QTMAKnobList knobList,long knobIndex)
	{
	QTMAKnobListRecord *qlr;

	qlr = *(QTMAKnobListRecord **)knobList;
	knobIndex--;
	if(knobIndex < 0)
		knobIndex = 0;
	else if(knobIndex >= qlr->knobCount)
		knobIndex = qlr->knobCount - 1;
	return &qlr->qk[knobIndex];
	}

void QTMAKnobRefToKnob(QTMAKnobRef ref,QTMAKnobList *knobList,long *knobIndex)
	{
	QTMAKnobRecord *qk;

	qk = (QTMAKnobRecord *)ref;

	if(knobList)
		*knobList = qk->knobList;
	if(knobIndex)
		*knobIndex = qk->knobIndex;
	}


void SetQTMAKnobSettings(QTMAKnobList knobList, long knobIndex,
		Handle stringList,Handle categoryLasts,Handle categoryNames)
/*
 * you can pass nil for categoryLasts _and_ categoryNames, if you want.
 * we leave all 3 handles locked.
 */
	{
	long stringCount,catCount;
	unsigned char *w;
	unsigned short *lastW;
	unsigned char *catW;
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	if(qk->mainType == kQKPopup)
		{
		QTMAKnobListRecord *qlr;
		long i;

		qlr = *(QTMAKnobListRecord **)qk->knobList;
		SetMCPopupMenu(&qk->mcp,nil);		/* start a fresh menu */
	
		HLock(stringList);
		stringCount = **(short **)stringList;
		w = (*(unsigned char **)stringList) + 2;

		if(categoryLasts)
			{
			HLock(categoryNames);
			HLock(categoryLasts);
			lastW = (*(unsigned short **)categoryLasts);
			catCount = *lastW++;
			catW = (*(unsigned char **)categoryNames) + 2;
			}
		else
			catCount = 0;

		i = qk->kd.lowValue;
		if(catCount)
			{
			AddMCPopupCategory(&qk->mcp,catW);
			catW += 1 + *catW;
			catCount--;
			}
			
		while(stringCount--)
			{
			AddMCPopupItem(&qk->mcp,w,i);
			w += 1 + *w;

			if(catCount && (i == *lastW))
				{
				lastW++;
				AddMCPopupCategory(&qk->mcp,catW);
				catW += 1 + *catW;
				catCount--;
				}
			i++;
			}
		}
	else if(qk->mainType == kQKBoolean)
		{
		long size;

		size = GetHandleSize(stringList);
		qk->boolTags = NewHandle(size);
		BlockMove(*stringList,*qk->boolTags,size);
		}
	}



void SetQTMAKnobActive(QTMAKnobList knobList,long knobIndex,Boolean active)
	{
	QTMAKnobRecord *qk;

	qk = GetKnobRecord(knobList,knobIndex);

	qk->active = active;
	}




short rGetMenuID(MenuHandle mH);


short rGetMenuID(MenuHandle mH)
	{
	return (**mH).menuID;
	}


void InitializeMCPopup(MCPopup *mcp,Rect *r,OddMenu om,long id,long choice)
	{
	GrafPort *gp;

	GetPort(&gp);

	mcp->r = *r;
	mcp->frameR = *r;
	mcp->id = id;
	InsetRect(&mcp->frameR,-kMCPopupBoxMargin,-kMCPopupBoxMargin);

	if(om)
		{
		mcp->om = om;
		mcp->ourMenu = false;
		mcp->choice = choice;
		}
	else
		{
		mcp->om = NewOddMenu(mcp->id++);
		mcp->ourMenu = true;
		mcp->choice = 0;
		}


	mcp->font = gp->txFont;
	mcp->size = gp->txSize;
	mcp->face = gp->txFace;
	}


void SetMCPopupChoice(MCPopup *mcp,long choice)
	{
	mcp->choice = choice;
	}

void SetMCPopupMenu(MCPopup *mcp,OddMenu om)
	{
	if(mcp->ourMenu)
		DisposeOddMenu(&mcp->om);

	if(om)
		{
		mcp->om = om;
		mcp->ourMenu = false;
		}
	else
		{
		mcp->om = NewOddMenu(mcp->id++);
		mcp->ourMenu = true;
		mcp->choice = 0;
		}
	}



void TerminateMCPopup(MCPopup *mcp)
	{

	if(mcp->ourMenu)
		{
		DisposeOddMenu(&mcp->om);
		}
	}

#define kElevation 0
#define kLeftMargin 1

void DrawMCPopup(MCPopup *mcp)
	{
	Str255 s;
	Rect r;
	UInt16 w,h;
	Fontystuff fs;

	GetFontystuff(&fs);

	TextFont(mcp->font);
	TextSize(mcp->size);
	TextFace(mcp->face);

	r = mcp->frameR;

	FrameRect(&r);

	MoveTo(r.right,r.top + 3);
	LineTo(r.right,r.bottom);
	LineTo(r.left+3,r.bottom);

	MoveTo(mcp->r.left + kLeftMargin,mcp->r.bottom - kElevation);

//	GetMenuItemText(mcp->mH, mcp->choice, s);
	s[0] = 1; s[1] = '3';
	GetOddMenuText(mcp->om,mcp->choice,s);

	InsetRect(&r,1,1);
	EraseRect(&r);
//	DrawString(s);
	w = mcp->r.right - mcp->r.left - (2*kLeftMargin);
	DrawStringTruncated(s, w);

	SetFontystuff(&fs);
	}

void GetMCPopupString(MCPopup *mcp,StringPtr s)
	{
	s[0] = 1; s[1] = '3';
	GetOddMenuText(mcp->om,mcp->choice,s);
	}

Boolean HitMCPopup(MCPopup *mcp,Point p)
	{
	return PtInRect(p,&mcp->frameR);
	}

long TrackMCPopup(MCPopup *mcp,Point p)
	{
	ClickMCPopup(mcp,p);
	return mcp->choice;
	}

Boolean ClickMCPopup(MCPopup *mcp,Point p)
	{
	Point globalKaka;
	long popResult;
	Boolean result;

	result = false;

	if(PtInRect(p,&mcp->frameR))
		{
		result = true;

		globalKaka.h = mcp->frameR.left+1;
		globalKaka.v = mcp->frameR.top+1;
		LocalToGlobal(&globalKaka);


			{
			GrafPtr oldPort,wPort;
			long oldSize,oldFace,oldFont;

			GetPort(&oldPort);
			GetWMgrPort(&wPort);
			SetPort(wPort);

			oldFace = wPort->txFace;
			oldFont = wPort->txFont;
			oldSize = wPort->txSize;

			TextFont(mcp->font);
			TextSize(mcp->size);
			TextFace(mcp->face);
			SetPort(oldPort);

			popResult = OddMenuSelect(mcp->om,globalKaka,mcp->choice);

			SetPort(wPort);
			TextFont(oldFont);
			TextSize(oldSize);
			TextFace(oldFace);
			SetPort(oldPort);
			}

//		popResult &= 0xFFFF;
		if(popResult != -1)
			{
			mcp->choice = popResult;
			DrawMCPopup(mcp);
			}
		}

	return result;
	}

short GetMCPopupChoice(MCPopup *mcp)
	{
	return mcp->choice;
	}

void BumpMCPopup(MCPopup *mcp,short bump)
	{
	OffsetRect(&mcp->r,0,bump);
	OffsetRect(&mcp->frameR,0,bump);
	}

long AddMCPopupItem(MCPopup *mcp,StringPtr s,long itemRef)
	{
	AddOddMenuItem(mcp->om,s,itemRef);

	return 0;
	}


long AddMCPopupCategory(MCPopup *mcp,StringPtr s)
	{
	AddOddMenuCategory(mcp->om,s);

	return 0;
	}



void DrawStringCenter(StringPtr s)
/*
 * Draw the string centered at
 * the current pen position; 
 * leave the pen where it started.
 */
	{
	Point po;

	GetPen(&po);
	Move(-StringWidth(s)/2,0);
	DrawString(s);
	MoveTo(po.h,po.v);
	}

void DrawNum(long n)
	{
	Str255 s;
	MyNumToString(n,0,s);
	DrawString(s);
	}


void DrawStringRight(StringPtr s)
/*
 * Draw the string from the current pen position
 * off to the left; leave the pen where it started.
 */
	{
	Point po;

	GetPen(&po);
	Move(-StringWidth(s),0);
	DrawString(s);
	}


void TruncateString(StringPtr s, short width)
/*
 * Draw the string limited to the width passed,
 * Truncated if necessary with a trailing .
 */
	{
	while(s[0] > 1 && StringWidth(s) > width)
		{
		s[0] --;
		s[s[0]] = '';
		}

	if(s[0] == 1 && StringWidth(s) > width)
	s[0] = 0;
	}


static void CopyPString(StringPtr dest,StringPtr src)
	{
	if(dest && src)
		BlockMove(src,dest,(*src) + 1);
	}

static void AppendPString(StringPtr dest,StringPtr src)
{
	int len;
	int chars_to_copy;
	if(dest && src)
	{
		chars_to_copy = src[0];
		if ((chars_to_copy == 0) || (dest[0] == 255))
			return;
		if (chars_to_copy + dest[0] >= 255)
			chars_to_copy = 255 - dest[0];
		for (len = 1; len <= chars_to_copy; len++)
			dest[dest[0]+len] = src[len];
		dest[0] += chars_to_copy;
	}
}

static void DrawStringTruncated(StringPtr s, short width)
/*
 * Draw the string limited to the width passed,
 * Truncated if necessary with a trailing .
 */
	{
	Str255 s2;

	CopyPString(s2,s);
	TruncateString(s2,  width);
	DrawString(s2);
	}






#define kOMMMax 64		/* number of menus, submenus max */
#define kOMIMax 520	/* number of items, total max */
#define kFirstCatMenuID (235 - kOMMMax)

static long gOMID = 23015;

typedef struct
	{
	MenuRef mH;
	short mID;
	} OM;

typedef struct
	{
	long menuItem;		/* menu id/menu item pairs */
	long menuItemRef;	/* the reference as seen by the user */
	MenuRef mH;			/* what menuhandle this item lives in. */
	short selectItem;	/* which item to start popup on if this one is active */ 
	} OI;

typedef struct
	{
	long menuCount;				/* how many menus total */
	OM om[kOMMMax];				/* menus */

	short categoryAdding;		/* which menu we're adding items to */
	MenuRef categoryAddingMH;	/* menuhandle of category we're adding to */

	long itemCount;
	OI oi[kOMIMax];				/* items */
	} OMR,**OMRH;


#define hlock(x) HLock((Handle)(x))
#define hunlock(x) HUnlock((Handle)(x))



static void InsertOddMenu(OMR *omr);
static void DeleteOddMenu(OMR *omr);
static OI *FindOddItem(OddMenu o,long itemRef);




void AddAnotherMenu(OMR *omr,StringPtr s,long id);		// id must be specified for first menu, may be zero after
void AddAnotherMenu(OMR *omr,StringPtr s,long id)
	{
	OM *om;

	if(omr->menuCount)
		id = kFirstCatMenuID + omr->menuCount;

	om = omr->om+omr->menuCount;
	om->mID = id;
	om->mH = NewMenu(id, s);

	omr->categoryAdding = omr->menuCount;
	omr->categoryAddingMH = om->mH;

	if(omr->menuCount != 0)		/* adding past the first one? insert it as hier menu */
		{
		Str255 s2;
		unsigned char *w;

		BlockMove(s,s2,s[0]+1);
		w = s2+s2[0]+1;
		*w++ = '/';
		*w++ = 0x1B;
		*w++ = '!';
		*w++ = id;
		s2[0] += 4;
		AppendMenu(omr->om[0].mH,s2);
		}

	omr->menuCount++;
	}


OddMenu NewOddMenu(long id)
	{
	OMRH omrH;

	omrH = (OMRH)NewHandleClear(sizeof(OMR));

	hlock(omrH);
	AddAnotherMenu(*omrH,"\p.",id);
	hunlock(omrH);

	return (OddMenu)omrH;
	}

void AddOddMenuItem(OddMenu o,StringPtr s,long itemRef)
	{
	OMRH omrH;
	OMR *omr;
	OM *om;
	OI *oi;

	omrH = (OMRH)o;
	hlock(omrH);
	omr = *omrH;

	om = omr->om + omr->categoryAdding;
	if(s)		/* adding a menu item */
		{
		AppendMenu(om->mH,"\p.");
		SetMenuItemText(om->mH,CountMItems(om->mH),s);
		oi = omr->oi + omr->itemCount++;
		oi->menuItem = (((long)om->mID)<<16) + CountMItems(om->mH);
		oi->menuItemRef = itemRef;
		oi->mH = omr->categoryAddingMH;
		oi->selectItem = CountMItems(omr->om[0].mH);
		}
	else
		AppendMenu(om->mH,"\p(-");

	hunlock(omrH);
	}

void EnableOddMenuItem(OddMenu o,long itemRef,Boolean enable)
	{
	OMRH omrH;
	OMR *omr;
	OM *om;
	OI *oi;
	short i;

	omrH = (OMRH)o;
	hlock(omrH);
	omr = *omrH;

	for(i = 0; i < omr->itemCount; i++)
		{
		oi = omr->oi + i;
		if(oi->menuItemRef == itemRef)
			{
			if(enable)
					EnableItem(oi->mH, oi->menuItem & 0x0000FFFF);
			else
					DisableItem(oi->mH, oi->menuItem & 0x0000FFFF);
			}
		}

	hunlock(omrH);
	}

void AddOddMenuCategory(OddMenu o,StringPtr s)			/* use nil or emptystring to be category-free again */
	{
	OMRH omrH;
	OMR *omr;
	OM *om;
	OI *oi;

	omrH = (OMRH)o;
	hlock(omrH);
	omr = *omrH;

	if(s && s[0])
		AddAnotherMenu(omr,s,0);
	else
		omr->categoryAdding = 0;

	hunlock(omrH);
	}

long OddMenuFindItemRef(OMR *omr,long selectResult);
long OddMenuFindItemRef(OMR *omr,long selectResult)
	{
	OI *oi;
	long i,result;

	result = -1;

	oi = omr->oi;
	for(i = 0; i < omr->itemCount; i++)
		{
		if(oi->menuItem == selectResult)
			{
			result = oi->menuItemRef;
			goto gotIt;
			}
		oi++;
		}
gotIt:
	return result;
	}

void InsertOddMenu(OMR *omr)
	{
	OM *om;
	long i;

	om = omr->om;
	for(i = 0; i < omr->menuCount; i++)
		{
		InsertMenu(om->mH,-1);
		om++;
		}
	}

void DeleteOddMenu(OMR *omr)
	{
	OM *om;
	long i;

	om = omr->om;
	for(i = 0; i < omr->menuCount; i++)
		{
		DeleteMenu(om->mID);
		om++;
		}
	}

void DisposeOddMenu(OddMenu *o)
	{
	OMRH omrH;
	OMR *omr;
	OM *om;
	long i;

	if(o && *o)
		{
		omrH = (OMRH)*o;

		hlock(omrH);

		omr = *omrH;

		om = omr->om;
		for(i = 0; i < omr->menuCount; i++)
			{
			DisposeMenu(om->mH);
			om++;
			}
	
		DisposeHandle((Handle)omrH);
		*o = 0;
		}
	}


long OddMenuSelect(OddMenu o,Point p,long choice)
	{
	OMRH omrH;
	OMR *omr;
	OM *om;
	OI *oi;
	long popResult;

	omrH = (OMRH)o;
	hlock(omrH);
	omr = *omrH;

	om = omr->om;

	InsertOddMenu(omr);

	oi = FindOddItem(o,choice);

	SetItemMark(oi->mH, oi->menuItem & 0xFFFF, checkMark);

	popResult = PopUpMenuSelect(om->mH,
			p.v,p.h,
			oi->selectItem);

	SetItemMark(oi->mH, oi->menuItem & 0xFFFF, noMark);

	DeleteOddMenu(omr);
	popResult = OddMenuFindItemRef(omr,popResult);
	hunlock(omrH);

	return popResult;
	}

OI *FindOddItem(OddMenu o,long itemRef)
	{
	OMRH omrH;
	OMR *omr;
	OI *oi;
	short i;

	omrH = (OMRH)o;
	omr = *omrH;

	oi = omr->oi;
	for(i = 0; i < omr->itemCount; i++)
		{
		if(oi->menuItemRef == itemRef)
			goto foundItem;
		oi++;
		}

	oi = 0;
foundItem:
	return oi;
	}

void GetOddMenuText(OddMenu o,long itemRef,StringPtr s)
	{
	OMRH omrH;
	OMR *omr;
	OI *oi;
	OM *om;
	short i;

	s[0] = 0;

	omrH = (OMRH)o;
	hlock(omrH);
	omr = *omrH;

	oi = FindOddItem(o,itemRef);
	if(oi)
		{
		om = omr->om;
		for(i = 0; i < omr->menuCount; i++)
			{
			if(om->mID == (oi->menuItem>>16) )
				goto foundMenu;
			om++;
			}
		goto goHome;
	
	foundMenu:
		GetMenuItemText(om->mH,oi->menuItem & 0xFFFF,s);
		}
goHome:;
	}


