 
/*--------------------------
	Inclusions
--------------------------*/

#include <QuickDraw.h>
#include <Windows.h>
#include <Resources.h>
#include <OSUtils.h>

#include "BigEasy2.h"
#include "BigEasyUtils.h"
#include "BigEasyTextish.h"
#include "BigEasyDialogs.h"

#include <QuickTimeComponents.h>

/*--------------------------
	Limits and Konstants
--------------------------*/
enum
	{
	mOpen = 100,
	mFirstEditField,
	mEditSynthesizerType,
	mEditSynthesizerName,
	mEditInstrumentName,
	mEditInstrumentNumber,
	mEditGMNumber,
	mMoveable,

	mNewNoteChannel,

	mPickInstrument,
	mPickInstrumentNoMix,
	mPickInstrumentSameSynth,
	mPickInstrumentFascist,
	mUserInsts,

	mEditGlobals,
	mEditInstrument,
	mPickEditInstrument,

	mClose
	};

typedef struct
	{
	WindowPtr w;

	NoteAllocator na;
	AtomicInstrument ai;

	Boolean moveable;		/* if true, let picker/editor be dragged */
	Boolean userInsts;
	} Globals;

Globals g;


/*--------------------------
	Prototypes
--------------------------*/
static void DrawDoc(short n);
static void ClickDoc(short n,Point p,short mods);
static void KeyDoc(short n,short key,short code,short mods);
static void IdleDoc(short n, Boolean front);
static void GoAwayDoc(short n);
static void ActivateDoc(short n);
static void DeactivateDoc(short n);
static void LetsQuit(short n,short menuItem,short menuRef);
static void EditFieldMenu(short n,short menuItem,short menuRef);

static void DocSeqTest(short n,short menuItem,short menuRef);
static void DocNoteChannel(short n,short menuItem,short menuRef);
static void Pick(short n,short menuItem,short menuRef);
static void OpenWindow(short n,short menuItem,short menuRef);
static void InitVars(void);
static void EditField(short f);
static void InvalFields(void);

static pascal Boolean MyFilterProc(DialogPtr theDialog,
		EventRecord *theEvent,short *itemHit);
	
MusicComponent GetNAMC(NoteAllocator na,NoteChannel nc,long *partOut);
ToneDescription *FindAtomicTD(AtomicInstrument ai,ToneDescription *tdOut);
Ptr FindAtomicThingie(AtomicInstrument ai,QTAtomType atomType,long size);
ComponentResult setAI(AtomicInstrument ai,ToneDescription *td,AtomicInstrument xx);

/*--------------------------
	Computer Programs
--------------------------*/

#define kLineHeight 12
#define kDocMargin 12

#define kLine1 30
#define kLine2 42
#define kLine3 54
#define kLine4 66
#define kLine5 78
#define kCenterLine 125

#define kDivideLine 84
#define kMoreLine1 98
#define kMoreLine2 110
#define kMoreLine3 122


#define kWindowWidth (kCenterLine*2)
#define kWindowHeight (kMoreLine3 + 6)

#define kHeaderHeight 17

void PrintOSType(OSType x);
void PrintOSType(OSType x)
	{
	DrawChar((x>>24) & 0xFF);
	DrawChar((x>>16) & 0xFF);
	DrawChar((x>>8) & 0xFF);
	DrawChar((x>>0) & 0xFF);
	}

void PrintAtomSortOf(QTAtomContainer ai,QTAtom atom);
void PrintAtomSortOf(QTAtomContainer ai,QTAtom atom)
	{
	QTAtom w;
	OSErr result;
	long dataSize;
	Ptr data;

	result = QTGetAtomDataPtr( ai, atom, &dataSize, &data );
	if(!result)
		{
		// atom has data
		DrawString("\p-");
		DrawNum(dataSize);
		}

		{
		QTAtomType at;
		long childCount,i;
		QTAtomID atomID;
		Boolean anyChildren = false;
	
		at = 0;
	
	doChildType:
		at = QTGetNextChildType( ai, atom, at );
		if(at)
			{
			DrawString("\p ");
			if(!anyChildren)
				{
				DrawString("\p(");
				anyChildren = true;
				}

			childCount = QTCountChildrenOfType( ai, atom, at );
			for(i = 1; i <= childCount; i++)
				{
				w = QTFindChildByIndex( ai, atom, at, i, &atomID );
				PrintOSType(at);
				PrintAtomSortOf(ai,w);
				}
			goto doChildType;
			}
		if(anyChildren)
			DrawString("\p)");
		}
	}


void DrawDoc(short n)
/*
 * Draws the window.
 */
	{
	#pragma unused (n)
	ToneDescription td;
	ComponentResult result;
	Str255 s;

	result = !FindAtomicTD(g.ai,&td);


	EraseRect(&gBigRect);
	TextSize(9);
	MoveTo(kCenterLine,12);
	DrawStringCenter("\pTone Description");
	MoveTo(0,kHeaderHeight-3);
	Line(9000,0);
	MoveTo(0,kHeaderHeight-1);
	Line(9000,0);

/* ---------------- */
	MoveTo(kCenterLine,kLine1);
	DrawStringRight("\pSynthesizer Type: ");
//	if(td.synthesizerType)
		{
		DrawChar('');
		DrawChar(td.synthesizerType>>24);
		DrawChar((td.synthesizerType>>16) & 0xFF);
		DrawChar((td.synthesizerType>>8) & 0xFF);
		DrawChar((td.synthesizerType) & 0xFF);
		DrawString("\p   0x");
		DrawHexLong(td.synthesizerType);
		}
//	else
//		DrawString("\p0");


/* ---------------- */
	MoveTo(kCenterLine,kLine2);
	DrawStringRight("\pSynthesizer Name: ");
	DrawString(td.synthesizerName);

/* ---------------- */
	MoveTo(kCenterLine,kLine3);
	DrawStringRight("\pInstrument Name: ");
	DrawString(td.instrumentName);

/* ---------------- */
	MoveTo(kCenterLine,kLine4);
	DrawStringRight("\pInstrument Number: ");
	DrawNum(td.instrumentNumber);

/* ---------------- */
	MoveTo(kCenterLine,kLine5);
	DrawStringRight("\pGM Number: ");
	DrawNum(td.gmNumber);



/* ---------------- */
		{
		PicHandle pH;
		Rect r;
		InstKnobList *ikl;

		QTLockContainer(g.ai);
		MoveTo(kDocMargin,kDivideLine);
		LineTo(kWindowWidth - kDocMargin,kDivideLine);

		pH = (PicHandle)GetResource('PICT', 128);
		if(pH)
			{
			r = (**pH).picFrame;
			OffsetRect(&r,kDocMargin - r.left,
					kDivideLine + ((kWindowHeight - kDivideLine) - (r.bottom - r.top))/2);
			DrawPicture(pH,&r);
			r.right += kDocMargin;
			}
		else
			r.right = kDocMargin;

		MoveTo(r.right,kMoreLine1);
		PrintAtomSortOf(g.ai,0);

		ikl = (InstKnobList *)FindAtomicThingie(g.ai,kaiKnobListType,0);
		if(ikl)
			{
			MoveTo(r.right,kMoreLine2);
			DrawString("\pKnob list of ");
			DrawNum(ikl->knobCount);
			DrawString("\p knobs.");
			}
	
		if(FindAtomicThingie(g.ai,kaiSampleDataType,0))
			{
			MoveTo(r.right,kMoreLine3);
			DrawString("\pSample data present.");
			}

		QTUnlockContainer(g.ai);
		}

	}



static void Pick(short n,short menuItem,short menuRef)
	{
	unsigned long flags;
	ToneDescription td;
	ComponentResult result;

	result = !FindAtomicTD(g.ai,&td);

	if(menuRef == mPickInstrumentNoMix)
		flags = kPickDontMix;
	else if(menuRef == mPickInstrumentSameSynth)
		flags = kPickSameSynth;
	else if(menuRef == mPickInstrumentFascist)
		flags = kPickDontMix + kPickSameSynth;
	else
		flags = 0;

	if(g.userInsts)
		flags |= kPickUserInsts;

	result = NAPickInstrument(g.na,g.moveable ? MyFilterProc : nil,"\pPick An Instrument:",&td,
			flags,0,0,0);

	if(!result)
		result = setAI(g.ai,&td,nil);

	InvalFields();
	}

static void Edit(short n,short menuItem,short menuRef)
/*
 * If control key, try passing nothing at all.
 */
	{
	unsigned long flags;
	NoteChannel nc;
	long index,part;
	MusicComponent mc;
	long xxx;
	Boolean passNC,passNothing;
	ComponentResult result;

	mc = 0;
	part = 0;

	passNothing = !!(GetModKeys() & controlKey);

	if(passNothing)
		nc = 0;
	else
		{
		QTLockContainer(g.ai);
		if(0)
			{
			NoteRequest nr;


			result = !FindAtomicTD(g.ai,&nr.tone);
			nr.info.flags = 0;
			nr.info.reserved = 0;
			nr.info.polyphony = 1;
			nr.info.typicalPolyphony = 0x00010000;
			result = NANewNoteChannel(g.na,&nr,&nc);
			}
		else
			{
			result = NANewNoteChannelFromAtomicInstrument(g.na,*g.ai,
					kSetAtomicInstCallerTosses,&nc);
			}
		QTUnlockContainer(g.ai);
		}

	xxx = (menuRef == mPickEditInstrument) || (menuRef == mPickInstrument)
			? kPickEditAllowPick : 0;

	xxx |= (menuRef == mPickEditInstrument) || (menuRef == mEditInstrument)
			|| (menuRef == mEditGlobals)
			? kPickEditAllowEdit : 0;

	xxx |= (menuRef == mEditGlobals)
			? kPickEditSynthGlobal : 0;

	result = NAPickEditInstrument(g.na,
			g.moveable ? MyFilterProc : nil,
			"\pEdit Instrument",
			0,
			nc,
			0,xxx);

	if(nc)
		{
		MusicComponent mc;
		long part;
		AtomicInstrument newAI;

		mc = GetNAMC(g.na,nc,&part);
		if(mc && part)
			{
			result = MusicGetPartAtomicInstrument(mc,part,&newAI,0);
			if(newAI)
				{
				setAI(g.ai,0,newAI);
				QTDisposeAtomContainer(newAI);
				}
			}

		result = NADisposeNoteChannel(g.na,nc);
		}

	InvalFields();
	}

void ClickDoc(short n,Point p,short mods)
/*
 * Come here for a click in the window.
 */
	{
	Str31 s;



	if(p.v < kLine1 && p.v > kLine1 - kLineHeight)
		EditField(1);
	else if(p.v < kLine2 && p.v > kLine2 - kLineHeight)
		EditField(2);
	else if(p.v < kLine3 && p.v > kLine3 - kLineHeight)
		EditField(3);
	else if(p.v < kLine4 && p.v > kLine4 - kLineHeight)
		EditField(4);
	else if(p.v < kLine5 && p.v > kLine5 - kLineHeight)
		EditField(5);
	else
		Pick(0,0,0);
	}


void EditField(short f)
/*
 * Edit a field, 1-5.
 */
	{
	Str31 s;
	ToneDescription td;
	ComponentResult result;

	result = !FindAtomicTD(g.ai,&td);

	switch(f)
		{
		case 1:
			if(td.synthesizerType)
				{
				s[0] = 4;
				s[1] = td.synthesizerType >> 24;
				s[2] = td.synthesizerType >> 16;
				s[3] = td.synthesizerType >> 8;
				s[4] = td.synthesizerType;
				}
			else
				{
				s[0] = 1;
				s[1] = '0';
				}
			EasyDialogGetString("\pTone Description",
					"\pNew value for synthesizer type:",
					s,4);
			if(s[0] == 1 && s[1] == '0')
				td.synthesizerType = 0;
			else
				{
				while(s[0] < 4)
					s[++s[0]] = ' ';
				td.synthesizerType = (((unsigned long)s[1]) << 24)
						+ (((unsigned long)s[2]) << 16)
						+ (((unsigned long)s[3]) << 8)
						+ (((unsigned long)s[4]));
				}
			goto inval;

		case 2:
			EasyDialogGetString("\pTone Description",
					"\pNew value for synthesizer name:",
					td.synthesizerName,31);
			goto inval;

		case 3:
			EasyDialogGetString("\pTone Description",
					"\pNew value for instrument name:",
					td.instrumentName,31);
			goto inval;

		case 4:
			EasyDialogGetNumber("\pTone Description",
					"\pNew value for instrument number:",
					&td.instrumentNumber);
			goto inval;

		case 5:
			EasyDialogGetNumber("\pTone Description",
					"\pNew value for GM number:",
					&td.gmNumber);
	inval:
			InvalFields();
			break;
		}

	result = setAI(g.ai,&td,nil);
	}

void InvalFields(void)
	{
	Rect r;

	SetPort(g.w);

	r.top = kHeaderHeight;
	r.left = kCenterLine;
	r.right = kWindowWidth;
	r.bottom = kDivideLine;
	InvalRect(&r);

	r.top = kDivideLine+1;
	r.left = 0;
	r.right = kWindowWidth;
	r.bottom = kWindowHeight;
	InvalRect(&r);
	}

void KeyDoc(short n,short key,short code,short mods)
	{
	#pragma unused (n,key,code,mods)

	if(key == ' ' || key == 13 || key == 3)
		Pick(0,0,0);
	else if(key >= '1' && key <= '5')
		EditField(key-'0');
	}

void IdleDoc(short n, Boolean front)
	{
	#pragma unused (n,front)
	}

void GoAwayDoc(short n)
/*
 * Close that window...
 */
	{
	UninstallWindow(n);
	}

void FixUpMenus(void);
void FixUpMenus(void)
	{
 	SetMenuItem(mClose,1,0,0,nil);				/* enable "Close" menu item		*/
	SetMenuItem(mOpen,-1,0,0,nil);				/* disable "Open" menu item		*/

	SetMenuItem(mMoveable,1,g.moveable ? 1 : -1,18,nil);
	SetMenuItem(mUserInsts,1,g.userInsts ? 1 : -1,18,nil);
	}

void ActivateDoc(short n)
	{
	FixUpMenus();
	}

void DeactivateDoc(short n)
	{
#pragma unused (n)
	SetMenuItem(mClose,-1,0,0,nil);				/* disable "Close" menu item	*/
	SetMenuItem(mOpen,1,0,0,nil);				/* enable "Open" menu item		*/
	SetMenuItem(mMoveable,-1,0,0,nil);
	}

void LetsQuit(short n,short menuItem,short menuRef)
	{
#pragma unused (n,menuItem,menuRef)
	gQuitApp++;
	}


static void MoveableDoc(short n,short menuItem,short menuRef)
	{
	switch(menuRef)
		{
		case mMoveable:
			g.moveable = !g.moveable;
			break;
		case mUserInsts:
			g.userInsts = !g.userInsts;
			break;
		}

	FixUpMenus();
	}

void EditFieldMenu(short n,short menuItem,short menuRef)
	{
	EditField(menuRef - mFirstEditField);
	}

ComponentResult setAI(AtomicInstrument ai,ToneDescription *td,AtomicInstrument xx)
/*
 * if td is not nil, slam it into ai as the only atom
 * else, copy xx into ai
 */
	{
	ComponentResult result;

	result = QTRemoveChildren( ai,0 );
	if(td)
		result = QTInsertChild( ai, 0, kaiToneDescType, 1, 1, sizeof(ToneDescription), td, nil );
	else
		result = QTInsertChildren( ai,0, xx );

	return result;
	}


MusicComponent GetNAMC(NoteAllocator na,NoteChannel nc,long *partOut)
	{
	long index,part;
	MusicComponent mc;
	ComponentResult result;

	mc = 0;
	result = NAGetNoteChannelInfo(na,nc,&index,&part);
	if(index && !result)
		result = NAGetRegisteredMusicDevice(na,index,nil,nil,nil,&mc);

	if(partOut)
		*partOut = part;
	return mc;
	}


Ptr FindAtomicThingie(AtomicInstrument ai,QTAtomType atomType,long size)
	{
	Ptr outp;
	OSErr result;
	QTAtom anAtom;

	anAtom = QTFindChildByIndex( ai, kParentAtomIsContainer, atomType, 1, nil );
	if (!anAtom)
		{
		result = -1;
		goto goHome;
		}

	result = QTGetAtomDataPtr( ai, anAtom, &size, &outp );
goHome:
	if(result)
		outp = 0;

	return outp;
	}


ToneDescription *FindAtomicTD(AtomicInstrument ai,ToneDescription *tdOut)
	{
	ToneDescription *td;
	td = (ToneDescription *)FindAtomicThingie(ai,kaiToneDescType,sizeof(ToneDescription));
	if(td && tdOut)
		*tdOut = *td;
	return td;
	}

static void DocSeqTest(short n,short menuItem,short menuRef)
	{
	ToneDescription td;
	ComponentResult result;
	long xxx;

	td.synthesizerType = td.synthesizerName[0] =
			td.instrumentName[0] = td.instrumentNumber = td.gmNumber = 0;

	xxx = kPickEditAllowPick + kPickEditAllowEdit;

	result = NAPickEditInstrument(g.na,
			g.moveable ? MyFilterProc : nil,
			"\pSelect No Instrument/No Instrument to finish.",
			0,
			0,
			g.ai,xxx);

	InvalFields();
	}


static void DocNoteChannel(short n,short menuItem,short menuRef)
	{
	NoteChannel nc;
	long dummy;
	short p;
	ComponentResult result;

	QTLockContainer(g.ai);
	result = NANewNoteChannelFromAtomicInstrument(g.na,*g.ai,0,&nc);
	QTUnlockContainer(g.ai);

	p = 36;
	while(!Button())
		{
		NAPlayNote(g.na,nc,p,128);
		Delay(15,&dummy);
		NAPlayNote(g.na,nc,p,0);
		p++;
		}

	result = NADisposeNoteChannel(g.na,nc);
	}




void OpenWindow(short n,short menuItem,short menuRef)
	{
#pragma unused (n,menuItem,menuRef)
	Rect r;

	SetRect(&r,0,0,kWindowWidth,kWindowHeight);
	OffsetRect(&r,20,40);

	g.w = InstallWindow(1,"\p",&r,0,wCopyDraw,
			DrawDoc,ClickDoc,KeyDoc,nil,
			ActivateDoc,DeactivateDoc,IdleDoc);
	}


#ifdef RegisterLocalComponents
Component RegisterNoteAllocator(void);
#endif

void InitVars()
/*
 * Called once at startup: yes, it
 * inits the vars.
 */
	{
	ToneDescription td;
	ComponentResult result;

	EnterMovies();

	g.moveable = true;
	g.userInsts = false;

	g.na = OpenDefaultComponent('nota',0);

	td.synthesizerType = 0;
	td.synthesizerName[0] = 0;
	td.instrumentName[0] = 0;
	td.instrumentNumber = 0;
	td.gmNumber = 0;

	result = QTNewAtomContainer(&g.ai);
	if(g.ai)
		result = setAI(g.ai,&td,nil);

#ifdef RegisterLocalComponents
	RegisterNoteAllocator();
#endif
	}

void Bootstrap()
	{
	InitVars();

/*** File Menu ***/
	InstallMenu("\pFile",nil,0);
	InstallMenuItem("\pQuit/Q",LetsQuit,0);

/*** Edit Menu ***/
	InstallEditMenu(nil,nil,nil,nil,nil);

/*** Picker Menu ***/
	InstallMenu("\pPicker",nil,0);
	InstallMenuItem("\pSynthesizer Type/1",EditFieldMenu,mEditSynthesizerType);
	InstallMenuItem("\pSynthesizer Name/2",EditFieldMenu,mEditSynthesizerName);
	InstallMenuItem("\pInstrument Name/3",EditFieldMenu,mEditInstrumentName);
	InstallMenuItem("\pInstrument Number/4",EditFieldMenu,mEditInstrumentNumber);
	InstallMenuItem("\pGM Number/5",EditFieldMenu,mEditGMNumber);
	InstallMenuItem("\p(-",0,0);
	InstallMenuItem("\pPick Instrument/P",Pick,mPickInstrument);
	InstallMenuItem("\pPick Instrument [No Mix]/7",Pick,mPickInstrumentNoMix);
	InstallMenuItem("\pPick Instrument [Same Synth]/8",Pick,mPickInstrumentSameSynth);
	InstallMenuItem("\pPick Instrument [Fascist]/9",Pick,mPickInstrumentFascist);
	InstallMenuItem("\p(-",0,0);
	InstallMenuItem("\pEdit Instrument/E",Edit,mEditInstrument);
	InstallMenuItem("\pPick & Edit Instrument/F",Edit,mPickEditInstrument);
	InstallMenuItem("\pPick Instrument/O",Edit,mPickInstrument);
	InstallMenuItem("\pEdit Globals/Y",Edit,mEditGlobals);
	InstallMenuItem("\p(-",0,0);
	InstallMenuItem("\pNew Note Channel/N",DocNoteChannel,mNewNoteChannel);
	InstallMenuItem("\pPick& Edit Atomic/A",DocSeqTest,0);
	InstallMenuItem("\p(-",0,0);
	InstallMenuItem("\pMoveable Dialogs/M",MoveableDoc,mMoveable);
	InstallMenuItem("\pShow User Insts",MoveableDoc,mUserInsts);

	OpenWindow(0,0,0);
	}

void Hatstrap()
/*
  * clean up
  */
	{
	}


pascal Boolean MyFilterProc(DialogPtr theDialog,
		EventRecord *theEvent,short *itemHit)
	{
	GrafPort *oldPort;

	GetPort(&oldPort);
	SetPort(theDialog);

	if(theEvent->what == updateEvt && (DialogPtr)theEvent->message != theDialog)
			HandleUpdateEvent(theEvent);

	SetPort(oldPort);
	return false;
	}
