/*
 * file: QTMusic Views.c
 *
 * david van brink
 *
 * A starting point for BigEasy programs
 *
 */
 
 
/*--------------------------
	Inclusions
--------------------------*/


#include "BigEasy2.h"
#include "BigEasyTextish.h"
#include "BigEasyGrafish.h"
#include "BigEasyDialogs.h"

#ifdef PrivateInterfaces
	#include "NoteAllocatorComponent.h"
#else
	#include <QuickTimeComponents.h>
#endif
#include "QTMusicGraphtricks.h"


/*--------------------------
	Limits and Konstants
--------------------------*/


enum
	{
	mFirst = 100,

	mFirstDocOnly,
	mLastDocOnly,

	mLast
	};

/*--------------------------
	Types and globals
--------------------------*/

#define kScrollbarWidth 16

//  Dimensions for part information list

#define kPartNumberW 16
#define kPartInstrumentW 100
#define kPartPolyW 16
#define kPartChannelW 16
#define kPartH 12

#define kDocGap 4

#define kPartTableW (kPartNumberW+kPartInstrumentW+kPartPolyW+kPartChannelW + 5*kDocGap)


#define kNCPartW 16
#define kNCInstrumentW 100
#define kNCSynthW 100
#define kNCSynthPartW 16
#define kNCSynthChannelW 16
#define kNCActiveW 16

#define kNCTableW (kNCPartW+kNCInstrumentW+kNCSynthW+		\
		kNCSynthPartW+kNCSynthChannelW+kNCActiveW+			\
		7*kDocGap)

#define kSynthW (2*kPartTableW+kDocGap)
#define kSynthH 60

typedef struct
	{
	Str31 name;
	long polyphony;
	long midiChannel;
	long instrumentNumber;
	} PartInfo;

typedef struct
	{
	MusicComponent mc;
	SynthesizerDescription sd;
	Str31 name;					/* name registered */
	SynthesizerConnections sc;	/* if a MIDI device */
	long count;			/* part count derived from description, insts & drums */
	PartInfo *pi;
	} SynthesizerInfo;

typedef struct
	{
	long count;
	SynthesizerInfo *si;
	} SynthesizerInfoList;

typedef struct
	{
	NoteChannel nc;
	long partNumber;
	Str31 partName;
	Str31 synthName;
	long synthPart;
	long synthChannel;
	Str15 active;
	MusicComponent mc;
	} NoteChannelInfo;

typedef struct
	{
	long visibleCount;
	long count;
	NoteChannelInfo *nci;
	} NoteChannelInfoList;

typedef struct
	{
	NoteAllocator na;
	WindowPtr w;

	SynthesizerInfoList sil;
	SynthesizerInfoList oldSil;
	NoteChannelInfoList ncil;

	long topPixel;
	long windowHeight, imageHeight;
	ControlHandle vScroller;

	} Globals;


Globals *g;




#define SignIt(x) ( (x)?1:-1)

/*--------------------------
	Prototypes
--------------------------*/
static void DrawDoc(short n);
static void ClickDoc(short n,Point p,short mods);
static void KeyDoc(short n,short key,short code, short mods);
static void ActivateDoc(short n);
static void DeactivateDoc(short n);
static void MoveDoc(short n);
static void ResizeDoc(short windowNum, Point *oldSize, Point *newSize, short modifiers);
static void IdleDoc(short n,Boolean front);
static void LetsQuit(void);
static void About(void);
static void InitVars(void);

Boolean MismatchBytes(void *a, void *b,unsigned long len);


void UpdateSynthesizerInfoList(void);
void DrawSynthesizerInfoList(long *vertical,SynthesizerInfoList *sil,SynthesizerInfoList *oldSil);
void DrawPartTable(long *vertical,SynthesizerInfo *si,SynthesizerInfo *oldSi);
void DrawSynthInfo(long *vertical,SynthesizerInfo *si,Boolean draw);

void DrawNoteChannelInfoList(long *vertical,NoteChannelInfoList *ncil,NoteChannelInfoList *oldNcil);

void SetupScrollbar(void);
void BumpVScroll(long vBump);
void ScrollToNew(void);
pascal void ScrollAction(ControlHandle cH, short part);

/*--------------------------
	Computer Programs
--------------------------*/

#define kDocHeaderHeight 17
#define kDocChannelHeight 30

#define kClientColumn 15
#define kSynthColumn 75
#define kChannelColumn 155
#define kPolyphonyColumn 175
#define kValidColumn 195

#define kSynthNumberColumn 5
#define kSynthTypeColumn 30
#define kSynthNameColumn 60

#define kSynthPartsColumn 150










void BuildSynthesizerInfoList(SynthesizerInfoList *sil);
void BuildSynthesizerInfo(SynthesizerInfo *si);
void BuildPartInfo(SynthesizerInfo *si,long index,PartInfo *pi);
void DisposeSynthesizerInfoList(SynthesizerInfoList *sil);
void DisposeSynthesizerInfo(SynthesizerInfo *si);

void BuildNoteChannelInfoList(NoteChannelInfoList *ncil);
void BuildNoteChannelInfo(long index,NoteChannelInfo *nci);
void DisposeNoteChannelInfoList(NoteChannelInfoList *ncil);


/* ---------------- ---------------- O ---------------- ---------------- */

void BuildSynthesizerInfoList(SynthesizerInfoList *sil)
	{
	long x;
	SynthesizerInfo *si;
	long i;
	ComponentResult thisError;

	x = NAGetRegisteredMusicDevice(g->na,
			0,nil,nil,nil,0);

	if(x < 0)
		x = 0;

	DisposeSynthesizerInfoList(sil);	/* empty out anything thats in here */

	sil->count = x;
	if(x)
		{
		sil->si =
				(SynthesizerInfo *)NewPtrClear(sizeof(SynthesizerInfo) * x);
		for(i = 1; i <= x; i++)
			{
			si = sil->si + i - 1;
			thisError = NAGetRegisteredMusicDevice(g->na,
					i,nil,si->name,&(si->sc),&(si->mc));
			if(thisError >= 0)
				BuildSynthesizerInfo(si);
			}
		}
	else
		sil->si = 0;
	}

void BuildSynthesizerInfo(SynthesizerInfo *si)
	{
	long i;

	MusicGetDescription(si->mc,&si->sd);

	si->count = si->sd.partCount;
	if(!(si->sd.flags & kSynthesizerMixedDrums))
		si->count += si->sd.drumPartCount;

	if(si->count)
		{
		si->pi = (PartInfo *)
				NewPtrClear(sizeof(PartInfo) * si->count);

		for(i = 1; i <= si->count; i++)
			BuildPartInfo(si,i,si->pi+i-1);
		}
	else
		si->pi = 0;
	}

void BuildPartInfo(SynthesizerInfo *si,long index,PartInfo *pi)
	{
	ComponentResult thisError;

	thisError = MusicGetPart(si->mc,index,&pi->midiChannel,&pi->polyphony);
	thisError = MusicGetPartName(si->mc,index,pi->name);
	pi->instrumentNumber = MusicGetPartInstrumentNumber(si->mc,index);
	}

void DisposeSynthesizerInfoList(SynthesizerInfoList *sil)
	{
	SynthesizerInfo *si;
	long i;

	if(sil->si)
		{
		for(i = 0; i < sil->count; i++)
			DisposeSynthesizerInfo(sil->si+i);
		DisposePtr((Ptr)sil->si);
		sil->si = 0;
		}
	sil->count = 0;
	}


void DisposeSynthesizerInfo(SynthesizerInfo *si)
	{
	if(si->pi)
		{
		DisposePtr((Ptr)si->pi);
		si->pi = 0;
		}
	si->count = 0;
	}

/* ---------------- ---------------- O ---------------- ---------------- */


#define kNCChunk 6

pascal ComponentResult NAGetIndNoteChannel(NoteAllocator na,long index, NoteChannel *nc)
	ComponentCallNow(31,8); // dont ask, dont tell.

void BuildNoteChannelInfoList(NoteChannelInfoList *ncil)
	{
	ComponentResult result;
	short i;

	DisposeNoteChannelInfoList(ncil);

	result = NAGetIndNoteChannel(g->na, 0, nil);
	if(result <= 0)
		result = 0;

	ncil->count = result;
	ncil->visibleCount = (result + kNCChunk - 1) / kNCChunk * kNCChunk;
	if(ncil->visibleCount < kNCChunk)
		ncil->visibleCount = kNCChunk;
	ncil->nci = (NoteChannelInfo *)NewPtrClear(ncil->visibleCount * sizeof(NoteChannelInfo));
	for(i = 1; i <= result; i++)
		BuildNoteChannelInfo(i,ncil->nci+i-1);

	}



void DisposeNoteChannelInfoList(NoteChannelInfoList *ncil)
	{
	if(ncil->nci)
		{
		DisposePtr((Ptr)ncil->nci);
		ncil->nci = nil;
		}
	
	ncil->count = 0;
	ncil->visibleCount = 0;
	}

void BuildNoteChannelInfo(long index,NoteChannelInfo *nci)
	{
	ComponentResult result;
	long synthIndex;

	result = NAGetIndNoteChannel(g->na,index,&nci->nc);
	if(result < 0)
		goto goHome;

	result = NAGetNoteChannelInfo(g->na,nci->nc,&synthIndex,&nci->partNumber);
	if(result)
		goto goHome;

	result = NAGetRegisteredMusicDevice(g->na,synthIndex,
			nil,nci->synthName,nil,&nci->mc);
	if(result < 0)
		goto goHome;

	if(nci->mc)
		{
		if(nci->synthName[0] == 0)
			{
			SynthesizerDescription sd;
	
			result = MusicGetDescription(nci->mc,&sd);
			if(result)
				goto goHome;
			BlockMove(sd.name,nci->synthName,32);
			}
	
		result = MusicGetPartName(nci->mc,nci->partNumber,nci->partName);
		if(result)
			goto goHome;
	
		result = MusicGetPart(nci->mc,nci->partNumber,&nci->synthChannel,nil);
		if(result)
			goto goHome;

		nci->active[0] = 1;
		nci->active[1] = '';
		}
	else
		{
		nci->partName[0] = 1;
		nci->partName[1] = '-';
		nci->synthName[0] = 1;
		nci->synthName[1] = '-';
		nci->synthChannel = 0;
		nci->active[0] = 1;
		nci->active[1] = '-';
		}

goHome:
	if(result)
		nci->nc = 0;
	}
 

/* ---------------- ---------------- O ---------------- ---------------- */




#define kTableColor 35000,60500,35000
#define kNCColor 55000,30500,40000



void DrawSynthInfo(long *vertical,SynthesizerInfo *si,Boolean draw)
	{
	long y,x;
	Rect tableR,r;
	StringPtr name;
	Handle h;

	tableR.left = kDocGap;
	tableR.right = tableR.left + kSynthW;
	tableR.top = *vertical;
	tableR.bottom = tableR.top + kSynthH;
	*vertical = tableR.bottom;

	if(draw)
		{
		RGBFore(kTableColor);
		PaintRect(&tableR);
		GoBW();
	
		r.top = tableR.top + kDocGap;
		r.bottom = r.top + kPartH;
		r.left = tableR.left + kDocGap;
		r.right = tableR.right - kDocGap;
	
		name = si->name;
		if(!name[0])
			name = si->sd.name;

		TextFace(bold);
		EngraveStringR2(&r,name,kTableColor);
		TextFace(0);
	
		r.top = r.bottom + kDocGap;
		r.bottom = tableR.bottom - kDocGap;
		Frame3d(&r,kTableColor);
		DrawStringR(&r,"\p");
	
		h = NewHandle(0);
	
		if(si->sd.flags & kSynthesizerUsesMIDIPort)
			HPrintS(h,"\pMIDI synthesizer, ");
		
		if(si->sd.flags & kSynthesizerDynamicVoice)
			HPrintS(h,"\pdynamic voice allocation, ");

		if(si->sd.flags & kSynthesizerDynamicChannel)
			HPrintS(h,"\pdynamic MIDI channel allocation, ");
		
		if(si->sd.flags & kSynthesizerMicrotone)
			HPrintS(h,"\pcan play microtones, ");
		
		if(si->sd.flags & kSynthesizerMixedDrums)
			HPrintS(h,"\pparts may play drums or instruments, ");
		
		if(si->sd.flags & kSynthesizerSoftware)
			HPrintS(h,"\psoftware synthesizer, ");
		
		if(si->sd.flags & kSynthesizerHardware)
			HPrintS(h,"\phardware (non-MIDI) synthesizer, ");
		
		if(si->sd.flags & kSynthesizerGMSuperset)
			HPrintS(h,"\pGeneral MIDI superset, ");
		
		if(si->sd.flags & kSynthesizerGM)
			HPrintS(h,"\pGeneral MIDI, ");

		if(si->sd.flags & kSynthesizerHogsSystemChannel)
			HPrintS(h,"\phordes system MIDI channel, ");
		
		if(si->sd.flags & kSynthesizerSlowSetPart)
			HPrintS(h,"\pslow instrument changes, ");
		
		SetHandleSize(h,GetHandleSize(h)-2);
	
		HLock(h);
		InsetRect(&r,kDocGap,0);
		TETextBox(*h, GetHandleSize(h), &r, 0);
	
		DisposeHandle(h);
		}
	}



void DrawPartTable(long *vertical,SynthesizerInfo *oldSi,SynthesizerInfo *si)
/*
 * if oldSi is nil, then draw everything.
 * else, draw only the part entries that have changed,
 * and slam new values into si.
 */
	{
	long y,x;
	long columns,partsPerColumn;
	long i,j;
	long firstColumnPart,lastColumnPart;
	Rect tableR,r;
	PartInfo *pi,*oldPi;

	y = *vertical;
	columns = (qd.thePort->portRect.right + kDocGap) / (kPartTableW + kDocGap);
	if(columns < 2)
		columns = 2;
	partsPerColumn = (si->count + columns - 1)/columns;

	x = kDocGap;
	firstColumnPart = 1;

	for(i = 0; i < columns; i++)
		{
		y = *vertical;

		lastColumnPart = firstColumnPart + partsPerColumn - 1;
		if(lastColumnPart > si->count)
			lastColumnPart = si->count;

		tableR.left = x;
		tableR.right = tableR.left + kPartTableW;
		tableR.left -= i ? kDocGap : 0;
		tableR.top = y;
		tableR.bottom = tableR.top + (1+partsPerColumn) * (kDocGap + kPartH) + kDocGap;

		if(!oldSi)
			{
			RGBFore(kTableColor);
			PaintRect(&tableR);
			GoBW();
			}
	
		r.left = x + kDocGap;
		r.right = r.left + kPartNumberW;
		r.top = y + kDocGap;
		r.bottom = r.top + kPartH;
	
		y = r.bottom;
	
		if(!oldSi)
			{
			DrawStringR2(&r,"\pPart");
			BumpRect(&r,kDocGap,kPartInstrumentW);
			DrawStringR2(&r,"\pInstrument");
			BumpRect(&r,kDocGap,kPartPolyW);
			DrawStringR2(&r,"\pPoly");
			BumpRect(&r,kDocGap,kPartChannelW);
			DrawStringR2(&r,"\pCh");
			}
		
		for(j = firstColumnPart; j <= lastColumnPart; j++)
			{
			pi = si->pi+j-1;
			if(oldSi)
				oldPi = oldSi->pi+j-1;
	
			r.left = x + kDocGap;
			r.right = r.left + kPartNumberW;
			r.top = y + kDocGap;
			r.bottom = r.top + kPartH;
		
			y = r.bottom;
	
			if(!oldSi)
				{
				Frame3d(&r,kTableColor);
				DrawChangedNumR(&r,nil,j);
				}
	
			BumpRect(&r,kDocGap,kPartInstrumentW);
	
			if(oldSi)
				DrawChangedNumStringR(&r,oldPi->name,pi->name,&oldPi->instrumentNumber,pi->instrumentNumber);
			else
				{
				Frame3d(&r,kTableColor);
				DrawChangedNumStringR(&r,nil,pi->name,nil,pi->instrumentNumber);
				}
	
			BumpRect(&r,kDocGap,kPartPolyW);
	
			if(oldSi)
				DrawChangedNumR(&r,&oldPi->polyphony,pi->polyphony);
			else
				{
				Frame3d(&r,kTableColor);
				DrawChangedNumR(&r,nil,pi->polyphony);
				}
				
	
			BumpRect(&r,kDocGap,kPartChannelW);
	
			if(oldSi)
				DrawChangedNumR(&r,&oldPi->midiChannel,pi->midiChannel);
			else
				{
				Frame3d(&r,kTableColor);
				DrawChangedNumR(&r,nil,pi->midiChannel);
				}
			}

		x += kPartTableW + kDocGap;
		firstColumnPart += partsPerColumn;
		}
	
	*vertical = tableR.bottom;
	}













void DrawSynthesizerInfoList(long *vertical,SynthesizerInfoList *sil,SynthesizerInfoList *oldSil)
	{
	long i,y;
	SynthesizerInfo *si,*oldSi;
	PartInfo *pi;

	TextSize(9);
	y = *vertical;

	for(i = 0; i < g->sil.count; i++)
		{
		si = sil->si + i;
		if(oldSil)
			oldSi = oldSil->si+i;
		else
			oldSi = nil;

		y += kDocGap;
		DrawSynthInfo(&y,si,oldSil == 0);
//		y += kDocGap;
		DrawPartTable(&y,oldSi,si);
		}
	*vertical = y;
	}












void DrawNoteChannelInfoList(long *vertical,
		NoteChannelInfoList *ncil,NoteChannelInfoList *oldNcil)
/*
 * if oldNcil is nil, then draw everything.
 * else, draw only the part entries that have changed,
 * and slam new values.
 */
	{
	long y,x;
	long j;
	Rect tableR,r;
	NoteChannelInfo *nci,*oldNci;

	y = *vertical;

	y += kDocGap;

	x = kDocGap;

	tableR.left = x;
	tableR.right = tableR.left + kNCTableW;
	tableR.top = y;
	tableR.bottom = tableR.top + (1+ncil->visibleCount) * (kDocGap + kPartH) + kDocGap;

	if(!oldNcil)
		{
		RGBFore(kNCColor);
		PaintRect(&tableR);
		GoBW();

		r = tableR;

		r.left = r.right;
		r.right = 20000;
		PaintRect(&r);			/* everything off to the right */

		r.bottom = r.top;
		r.left = 0;
		r.top = *vertical;
		PaintRect(&r);			/* narrow sliver on top */
		}

	r.left = x + kDocGap;
	r.right = r.left + kNCPartW;
	r.top = y + kDocGap;
	r.bottom = r.top + kPartH;

	y = r.bottom;

	if(!oldNcil)
		{
		DrawStringR2(&r,"\pNC");
		BumpRect(&r,kDocGap,kNCInstrumentW);
		DrawStringR2(&r,"\pInstrument");
		BumpRect(&r,kDocGap,kNCSynthW);
		DrawStringR2(&r,"\pSynthesizer");
		BumpRect(&r,kDocGap,kNCSynthPartW);
		DrawStringR2(&r,"\pPart");
		BumpRect(&r,kDocGap,kNCSynthChannelW);
		DrawStringR2(&r,"\pCh");
		BumpRect(&r,kDocGap,kNCActiveW);
		DrawStringR2(&r,"\p");
		}
	
	for(j = 1; j <= ncil->visibleCount; j++)
		{
		nci = ncil->nci+j-1;
		if(oldNcil)
			oldNci = oldNcil->nci+j-1;

		r.left = x + kDocGap;
		r.right = r.left + kNCPartW;
		r.top = y + kDocGap;
		r.bottom = r.top + kPartH;
	
		y = r.bottom;

		if(!oldNcil)
			{
			Frame3d(&r,kNCColor);
			DrawChangedNumR(&r,nil,j <= ncil->count ? j : 0);
			}
		else
			{
			long oldJ,newJ;

			oldJ = j <= oldNcil->count ? j : 0;
			newJ = j <= ncil->count ? j : 0;
			DrawChangedNumR(&r,&oldJ,newJ);
			}

		BumpRect(&r,kDocGap,kNCInstrumentW);

		if(oldNcil)
			DrawChangedStringR(&r,oldNci->partName,nci->partName);
		else
			{
			Frame3d(&r,kNCColor);
			DrawChangedStringR(&r,nil,nci->partName);
			}

		BumpRect(&r,kDocGap,kNCSynthW);

		if(oldNcil)
			DrawChangedStringR(&r,oldNci->synthName,nci->synthName);
		else
			{
			Frame3d(&r,kNCColor);
			DrawChangedStringR(&r,nil,nci->synthName);
			}

		BumpRect(&r,kDocGap,kNCSynthPartW);

		if(oldNcil)
			DrawChangedNumR(&r,&oldNci->synthPart,nci->synthPart);
		else
			{
			Frame3d(&r,kNCColor);
			DrawChangedNumR(&r,nil,nci->synthPart);
			}

		BumpRect(&r,kDocGap,kNCSynthChannelW);

		if(oldNcil)
			DrawChangedNumR(&r,&oldNci->synthChannel,nci->synthChannel);
		else
			{
			Frame3d(&r,kNCColor);
			DrawChangedNumR(&r,nil,nci->synthChannel);
			}
	
		BumpRect(&r,kDocGap,kNCActiveW);

		if(oldNcil)
			DrawChangedStringR(&r,oldNci->active,nci->active);
		else
			{
			Frame3d(&r,kNCColor);
			DrawChangedStringR(&r,nil,nci->active);
			}
		}

	if(oldNcil)
		oldNcil->count = ncil->count;
	
	*vertical = tableR.bottom;
	}




void UpdateSynthesizerInfoList(void)
	{
	SynthesizerInfoList newSil;
	NoteChannelInfoList newNcil;
	SynthesizerInfo *si,*newSi;
	long i,j;
	long y;

	newSil.count = 0;
	newSil.si = 0;
	BuildSynthesizerInfoList(&newSil);

	newNcil.count = 0;
	newNcil.nci = 0;
	BuildNoteChannelInfoList(&newNcil);

	if(newSil.count != g->sil.count)
		goto completelyDifferent;

	for(i = 0; i < newSil.count; i++)
		{
		si = g->sil.si+i;
		newSi = newSil.si+i;

		if(si->mc != newSi->mc
				|| MismatchBytes(&si->sd,&newSi->sd,sizeof(SynthesizerDescription))
				|| MismatchBytes(&si->sc,&newSi->sc,sizeof(SynthesizerConnections))
				|| si->count != newSi->count)
			goto completelyDifferent;
		}

	if(newNcil.visibleCount != g->ncil.visibleCount)
		goto completelyDifferent;

somewhatDifferent:
	y = -g->topPixel;

	DrawSynthesizerInfoList(&y,&newSil,&g->sil);
	DisposeSynthesizerInfoList(&newSil);

	DrawNoteChannelInfoList(&y,&newNcil,&g->ncil);
	DisposeNoteChannelInfoList(&newNcil);

	goto goHome;

completelyDifferent:
	y = -g->topPixel;
	DisposeSynthesizerInfoList(&g->sil);
	g->sil = newSil;
	DisposeNoteChannelInfoList(&g->ncil);
	g->ncil = newNcil;

	DrawSynthesizerInfoList(&y,&g->sil,nil);
	DrawNoteChannelInfoList(&y,&g->ncil,nil);
	g->imageHeight = y + g->topPixel + kDocGap;

		{
		Rect r;

		r = gBigRect;
		r.top = y;
		PaintRect(&r);
		}

	SetupScrollbar();
goHome:;
	}

Boolean MismatchBytes(void *a, void *b,unsigned long len)
	{
	Boolean result;
	char *x,*y;

	x = (char *)a;
	y = (char *)b;

	while(len--)
		if(*y++ != *x++)
			return true;

	return false;
	}



void DrawDoc(short n)
/*
 * Draws the window.
 */
	{
	long y;
	GoBW();
	PaintRect(&gBigRect);

	y = -g->topPixel;
	DrawSynthesizerInfoList(&y,&g->sil,nil);
	DrawNoteChannelInfoList(&y,&g->ncil,nil);
	}

#define kSmallBump 10
#define kLargeBump 144

pascal void ScrollAction(ControlHandle cH, short part)
	{
	long oldValue,bump;
	long largeBump;

	largeBump = g->w->portRect.bottom;

	oldValue = GetCtlValue(cH);
	switch(part)
		{
		case inUpButton:
			bump = -kSmallBump;
			break;
		case inDownButton:
			bump = kSmallBump;
			break;
		case inPageUp:
			bump = - largeBump;
			break;
		case inPageDown:
			bump = largeBump;
			break;
		default:
			bump = 0;
		}

	SetCtlValue(cH,bump+oldValue);
	ScrollToNew();
	}



void ClickDoc(short n,Point p,short mods)
	{
	ControlHandle cH;
	short part;

	part = FindControl(p,g->w,&cH);
	if(cH == g->vScroller)
		{
		if(part == inThumb)
			{
			TrackControl(cH,p,nil);
			ScrollToNew();
			}
		else
			//TrackControl(cH,p,nil);
			TrackControl(cH,p,ScrollAction);
		}
	else
		InvalRect(&gBigRect);
	}

void KeyDoc(short n,short key,short code, short mods)
	{
	long vBump;
	Rect r;
	RgnHandle rH;

	vBump = 0;

	switch(key)
		{
		case 30:		// up arrow
			vBump = -20;
			break;
		case 31:		// down arrow
			vBump = 20;
			break;
		}

	if(vBump)
		BumpVScroll(vBump);
	}


void MoveDoc(short n)
	{
	}

static void ResizeDoc(short windowNum, Point *oldSize, Point *newSize, short modifiers)
	{
	InvalRect(&gBigRect);
	newSize->h = oldSize->h;
	SizeWindow(g->w,newSize->h,newSize->v,1);
	g->windowHeight = newSize->v;
	SetupScrollbar();
	}


void ActivateDoc(short n)
	{
	}

void DeactivateDoc(short n)
	{
	}

void IdleDoc(short n,Boolean front)
	{
	UpdateSynthesizerInfoList();
	}




void SetupScrollbar(void)
/*
 * Install the scrollbar according
 * the current window size & position
 */
	{
	Rect r;
	long min,max;


	r = qd.thePort->portRect;

	g->windowHeight = r.bottom - r.top;

	min = 0;
	max = g->imageHeight - g->windowHeight;
	if(max < 0)
		max = 0;

	r.left = r.right - kScrollbarWidth;
	r.bottom += 2-kScrollbarWidth;
	r.top -= 1;

	OffsetRect(&r,1,0);

	if(g->vScroller)
		{
		HideControl(g->vScroller);
		MoveControl(g->vScroller,r.left,r.top);
		SizeControl(g->vScroller,
				r.right - r.left,r.bottom - r.top);
		SetControlMaximum(g->vScroller,max);
		SetControlValue(g->vScroller,g->topPixel);
		ShowControl(g->vScroller);
		}
	else
		g->vScroller = NewControl(g->w, &r, "\p", true, g->topPixel, min, max,
				scrollBarProc, 0);

	BumpVScroll(0);
	}

void BumpVScroll(long vBump)
	{
	SetControlValue(g->vScroller,GetControlValue(g->vScroller) + vBump);

	ScrollToNew();
	}

void ScrollToNew(void)
	{
	long oldTopPixel;
	Rect r;
	RgnHandle rH;
	long vBump;

	oldTopPixel = g->topPixel;
	g->topPixel = GetControlValue(g->vScroller);
	vBump = g->topPixel - oldTopPixel;

	r = qd.thePort->portRect;
	r.right -= kScrollbarWidth-1;

	rH = NewRgn();
	ScrollRect(&r,0,-vBump,rH);
	InvalRgn(rH);

	/*
	 * Make sure it all scrolled, too.
	 */
	CopyRgn(((WindowPeek)(g->w))->updateRgn,rH);
	OffsetRgn(rH,0,-vBump);
	UnionRgn(rH,((WindowPeek)(g->w))->updateRgn,((WindowPeek)(g->w))->updateRgn);
	DisposeRgn(rH);
	}

void LetsQuit(void)
	{
	UninstallWindow(1);
	CloseComponent(g->na);

	gQuitApp++;
	}


void About(void)
	{
	EasyDialogMessage(0,(StringPtr)0x910,
			"\pby David Van Brink",
			kEasyDialogOkay);

	}

void InitVars(void)
/*
 * Called once at startup: yes, it
 * inits the vars.
 */
	{
	Rect r;

	g = (Globals *)NewPtrClear(sizeof(Globals));

	g->na = OpenDefaultComponent('nota',0);

	gStaggerWindows = true;
	SetRect(&r,0,0,2*kPartTableW + 3*kDocGap + kScrollbarWidth - 2,340);
	OffsetRect(&r,10,34);

	g->w = InstallWindow(1,"\pQuickTime Music",&r,0,wGrowable,
			DrawDoc,ClickDoc,KeyDoc,nil,
			ActivateDoc,DeactivateDoc,IdleDoc);

	SetupScrollbar();
	SetWindowMoveProc(1,MoveDoc);
	SetWindowResizeProc(1,ResizeDoc);
	}

void Bootstrap()
	{
/*** File Menu ***/
	InstallMenu("\pFile",nil,0);
	InstallPrintItems(nil,nil);
	InstallMenuItem("\p(-",nil,0);
	InstallQuitItem((void *)LetsQuit,0);

/*** Edit Menu ***/
	InstallEditMenu(nil,nil,nil,nil,nil);


		{
		Str63 s;

		CopyPString(s,"\pAbout ");
		ConcatenatePStrings(s,(StringPtr)0x910);
		ConcatenatePStrings(s,"\p");
		SetAbout(s,0,About);
		}

	SetMasterQuitAppProc(LetsQuit);

	InitVars();
	}

void Hatstrap()
/*
  * clean up
  */
	{
	}
