/*
	File:		Music Configuration.c

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <9>	11/14/95	dvb		Conditionally, use public interfaces.
		 <8>	11/13/95	dvb		correct MIDI channel
		 <7>	10/19/95	dvb		Get rid of ridiculous 16-checkbox midi channel selector.
		 <6>	 6/14/95	JB		added prototypes
		 <5>	 6/12/95	dvb		
		 <4>	 28-4-95	dvb		MIDI Input Selection
		 <3>	  6-4-95	dvb		handle errors on unopenable components
		 <2>	  8-3-95	dvb		Better support for OMS ports
		 <7>	11-10-94	dvb		
		 <6>	 22-9-94	dvb		
		 <5+>	 grrr use private headers again
		 <5>	 17-8-94	dvb		Support bigeasy printing.
		 <4>	 27-7-94	dvb		wider columns.
		 <3>	 6/10/94	dvb		Don't show Note Allocator as an output or an input for the user.
		 <2>	 5/27/94	dvb		.

*/

/*
 * file: Music Configuration.c
 *
 * david van brink
 *
 *
 */
 
 
/*--------------------------
	Inclusions
--------------------------*/

#include <QuickDraw.h>
#include <Windows.h>
#include <OSUtils.h>
#include <TextUtils.h>
#include <Memory.h>

#include "BigEasy2.h"
#include "BigEasyTextish.h"

#define _Globals_
#include "QTMusicConfiguration.h"
#undef _Globals_

/*--------------------------
	Limits and Konstants
--------------------------*/
enum
	{
	mOpen = 100,
	mClose,
	mSave,
	mAddSynth,
	mRemoveSynth,
	mSynthesizerInfo,
	mLastSynthesizerInfo = mSynthesizerInfo + kMaxMusicDevices,
	mNew
	};


/*--------------------------
	Prototypes
--------------------------*/
void UpdateSynthMenu(void);
void UpdateDeviceMenu(OneRegisteredDevice *ord);
void UnformatOneRegisteredDevice(OneRegisteredDevice *ord);
short FormatOneRegisteredDevice(OneRegisteredDevice *ord,short y);
void BumpOrdsTo(long bump);
short WhichSynth(OSType synthType,OneDevice **odOut);
void ZeroSC(SynthesizerConnections *sc);
Boolean EqualSC(SynthesizerConnections *sc1,SynthesizerConnections *sc2);


extern void DrawOneRegisteredDevice(OneRegisteredDevice *ord);



void PositionScrollBar(void);
void CloseInfo(short n);
Boolean OrdInUse(OneRegisteredDevice *ord);
ComponentResult UnregisterIfOrdInUse(OneRegisteredDevice *ord);
ComponentResult RegisterIfOrdInUse(OneRegisteredDevice *ord);
void SetOrdSynth(OneRegisteredDevice *ord, short synthMenuIndex);
void SetOrdPort(OneRegisteredDevice *ord, short portMenuIndex);
void SetOrdDevice(OneRegisteredDevice *ord, short deviceNumber);
void SetOrdMIDIChannel(OneRegisteredDevice *ord, short midiChannel);
void SetOrdName(OneRegisteredDevice *ord);
void InvalBelow(short top);
void AddSynth(short n,short menuItem,short menuRef);
void RemoveSynth(short n,short menuItem,short menuRef);
void BumpOrdTo(OneRegisteredDevice *ord,long bump);
void BumpOrdsTo(long bump);
pascal void ScrollAction(ControlHandle cH, short part);
void ClickDoc(short n,Point p,short mods);
void KeyDoc(short n,short key,short code,short mods);
void ActivateDoc(short n);
void DeactivateDoc(short n);
void LetsQuit(short n,short menuItem,short menuRef);
void SaveConfig(short n,short menuItem,short menuRef);
void IdleDoc(short n, Boolean front);
void SynthesizerInfo(short n,short menuItem,short menuRef);
static void ResizeDoc(short windowNum, Point *oldSize, Point *newSize, short modifiers);
void OpenWindow(short n,short menuItem,short menuRef);
void FindSynthesizers(void);
void InitVars(void);
void FindMIDIPorts(void);
void UpdateDeviceMenu(OneRegisteredDevice *ord);
void UpdateSynthMenu(void);
short WhichSynth(OSType synthType,OneDevice **odOut);
short WhichOrdSynth(OneRegisteredDevice *ord);
short WhichPort(OneRegisteredDevice *ord);
void FigureHases(OneRegisteredDevice *ord);
void UnformatOneRegisteredDevice(OneRegisteredDevice *ord);
short FormatOneRegisteredDevice(OneRegisteredDevice *ord,short y);
void BuildRegisteredDeviceList(long y);
void Bootstrap(void);
void Hatstrap(void);
short GetModKeys(void);
void ZeroSC(SynthesizerConnections *sc);
Boolean EqualSC(SynthesizerConnections *sc1,SynthesizerConnections *sc2);
	

/*--------------------------
	Computer Programs
--------------------------*/

void PositionScrollBar(void)
	{
	Rect r;
	short range;

	if(g->registeredDeviceCount)
		range = g->registeredDevice[g->registeredDeviceCount-1].ordBottom - kHeaderHeight + g->currentBump;
	else
		range = 0;

	range -= qd.thePort->portRect.bottom - kHeaderHeight;
	if(range < 0)
		range = 0;

	r.top = kHeaderHeight-3;
	r.right = kConfigWindowWidth+1;
	r.left = kConfigWidth;
	r.bottom = qd.thePort->portRect.bottom-14;

	if(!g->scrollBar)
		g->scrollBar = NewControl(g->w, &r, "\p",
				true,0,0,range,scrollBarProc,0);
	else
		{
		long x;
		if(r.left != g->scrollBarR.left
				|| r.right != g->scrollBarR.right
				|| r.top != g->scrollBarR.top
				|| r.bottom != g->scrollBarR.bottom)
			{
			HideControl(g->scrollBar);
			MoveControl(g->scrollBar, r.left,r.top);
			SizeControl(g->scrollBar, r.right-r.left,r.bottom-r.top);
			ShowControl(g->scrollBar);
			g->scrollBarR = r;
			}
		//range = 4000;
		SetControlMaximum(g->scrollBar, range);	

		x = GetCtlValue(g->scrollBar);
		BumpOrdsTo(x);
		}		
	}

void CloseInfo(short n)
	{
	OneDevice *io;

	UninstallWindow(n);

	n -= kFirstInfoWindow;
	io = &g->device[n];

	io->w = 0;
	}

Boolean OrdInUse(OneRegisteredDevice *ord)
	{
	short synthIndex,portIndex;
	OneDevice *od;
	Boolean result;

	result = true;

	synthIndex = GetMCPopupChoice(&ord->kindPopup);
	od = &g->device[synthIndex-1];

	if(od->sd.flags & kSynthesizerUsesMIDIPort)
		{
		if((!ord->sc.clientID) || (!ord->sc.midiChannel))
			result = false;
		}
	else if(od->sd.flags & kSynthesizerHardware)
		{
		if(!ord->sc.midiChannel)
			result = false;
		}
	else
		{
		ord->sc.clientID = 0;
		ord->sc.inputPortID = 0;
		ord->sc.outputPortID = 0;
		}

	return result;
	}

ComponentResult UnregisterIfOrdInUse(OneRegisteredDevice *ord)
/*
 * Remove the device from the NoteAllocator, and remove format data of it
 */
	{
	ComponentResult result;

	if(ord->sc.unique)
		result = NAUnregisterMusicDevice(g->na,ord->sc.unique);

	ord->sc.unique = 0;

	return result;
	}

ComponentResult RegisterIfOrdInUse(OneRegisteredDevice *ord)
/*
 * and format it regardless
 */
	{
	ComponentResult result;

	result = 0;

	if(OrdInUse(ord))
		{
		result = NARegisterMusicDevice(g->na,
			ord->synthType,ord->synthName,&ord->sc);

		result = NAGetRegisteredMusicDevice(g->na,ord->sc.unique,
				&ord->synthType,ord->synthName,&ord->sc,&ord->mc);
		}
	else
		ord->sc.unique = 0;

	return result;
	}

void SetOrdSynth(OneRegisteredDevice *ord,short synthMenuIndex)
/*
 * Set the synth, but dont reformat, since format-from
 * will be used, since our height might change
 */
	{
	OneDevice *od;
	ComponentResult result;

	od = &g->device[synthMenuIndex-1];
	GoORDClip();
	ActivateTEField(&g->teList,-1,true);
	GoBigClip();

	result = UnregisterIfOrdInUse(ord);
	UnformatOneRegisteredDevice(ord);

	CopyPString(ord->synthName,od->sd.name);
	ord->synthType = od->sd.synthesizerType;

	if(od->deviceMenu)
		ord->sc.midiChannel = 0;		/* if there's a choice of hardware, choose none for now */
	else if(od->hardwareDeviceCount)
		ord->sc.midiChannel = 1;

	result = RegisterIfOrdInUse(ord);

	GoORDClip();
	DrawTEField(&g->teList,ord->synthNameTEID);
	ActivateTEField(&g->teList,ord->synthNameTEID,true);
	GoBigClip();

	FormatOneRegisteredDevice(ord,ord->ordTop);
	DrawOneRegisteredDevice(ord);
	ActivateTEField(&g->teList,ord->synthNameTEID,true);
	}
	

void SetOrdPort(OneRegisteredDevice *ord,short portMenuIndex)
/*
 * Set the port, and reformat as appropriate
 */
	{
	QTMIDIPort *op;
	ComponentResult result;

	op = &g->port[portMenuIndex-1];

	result = UnregisterIfOrdInUse(ord);

		{
		short midiChannel;

		midiChannel = ord->sc.midiChannel;
		ord->sc = op->portConnections;
		ord->sc.midiChannel = midiChannel;
		}

	result = RegisterIfOrdInUse(ord);
	}

void SetOrdDevice(OneRegisteredDevice *ord,short deviceNumber)
	{
	ComponentResult result;

	result = UnregisterIfOrdInUse(ord);

	ord->sc.midiChannel = deviceNumber;
	result = RegisterIfOrdInUse(ord);
	}

void SetOrdMIDIChannel(OneRegisteredDevice *ord,short midiChannel)
	{
	ComponentResult result;

	result = UnregisterIfOrdInUse(ord);
	ord->sc.midiChannel = midiChannel;
	result = RegisterIfOrdInUse(ord);
	}

void SetOrdName(OneRegisteredDevice *ord)
	{
	ComponentResult result;

	result = UnregisterIfOrdInUse(ord);
	result = RegisterIfOrdInUse(ord);
	}

void InvalBelow(short top)
	{
	Rect r;

	r.top = top;
	r.left = 0;
	r.right = kConfigWidth;
	r.bottom = 10000;
	InvalRect(&r);
	}



void AddSynth(short n,short menuItem,short menuRef)
	{
	OneDevice *od;
	OneRegisteredDevice *ord;
	short y;

	if(g->registeredDeviceCount)
		y = g->registeredDevice[g->registeredDeviceCount-1].ordBottom;
	else
		y = kHeaderHeight + kDocMargin;

	ord = &g->registeredDevice[g->registeredDeviceCount];

	g->registeredDeviceCount++;

	ord->sc.clientID = 0;
	ord->sc.inputPortID = 0;
	ord->sc.outputPortID = 0;
	ord->sc.midiChannel = 1;
	ord->sc.flags = 0;
	ord->sc.unique = 0;
	ord->sc.reserved1 = 0;
	ord->sc.reserved2 = 0;

	od = &g->device[g->defaultDevice];
	ord->synthType = od->sd.synthesizerType;
	CopyPString(ord->synthName,od->sd.name);

	FormatOneRegisteredDevice(ord,y);

	InvalBelow(y);
	PositionScrollBar();
	DrawHeader();
	ActivateTEField(&g->teList,ord->synthNameTEID,true);
	}

void RemoveSynth(short n,short menuItem,short menuRef)
	{
	ComponentResult result;
	OneRegisteredDevice *ord;
	short top;

	if(g->registeredDeviceCount)
		{
		g->registeredDeviceCount--;
		ord = &g->registeredDevice[g->registeredDeviceCount];
		top = ord->ordTop;
		result = UnregisterIfOrdInUse(ord);
		UnformatOneRegisteredDevice(ord);
		PositionScrollBar();
		InvalBelow(top);
		DrawHeader();
		}
	}



void BumpOrdTo(OneRegisteredDevice *ord,long bump)
/*
 * bump value is how much UPWARDS we've been scrolled.
 * if g->currentBump = 5, then the tops of everything is
 * 5 pixels higher than the default
 */
	{
	bump = g->currentBump - bump;
	if(bump)
		{
		ord->ordTop += bump;
		ord->ordBottom += bump;
		GoORDClip();
		BumpMCPopup(&ord->kindPopup,bump);
		BumpTEField(&g->teList,ord->synthNameTEID,bump);
		if(ord->hasMIDIBox)
			{
			BumpMCPopup(&ord->portPopup,bump);
			BumpMCPopup(&ord->channelPopup,bump);
			}
		if(ord->hasDeviceBox)
			BumpMCPopup(&ord->devicePopup,bump);

		GoBigClip();
		}
	}

void BumpOrdsTo(long bump)
	{
	Rect r;
	RgnHandle oldClipRgn,trashedRgn;
	short i;

	if(bump != g->currentBump)
		{
		r.top = kHeaderHeight;
		r.bottom = qd.thePort->portRect.bottom;
		r.left = 0;
		r.right = kConfigWidth;
	
		trashedRgn = NewRgn();
		ScrollRect(&r,0,g->currentBump - bump,trashedRgn);
//		ValidRgn(trashedRgn);
		
		for(i = 0; i < g->registeredDeviceCount; i++)
			BumpOrdTo(g->registeredDevice+i,bump);
	
		oldClipRgn = NewRgn();
		GetClip(oldClipRgn);
		SetClip(trashedRgn);
		DrawDoc(1);
		SetClip(oldClipRgn);
	
		DisposeRgn(oldClipRgn);
		DisposeRgn(trashedRgn);
		g->currentBump = bump;
		}
	}

#define kSmallBump 10

pascal void ScrollAction(ControlHandle cH, short part)
	{
	long oldValue,bump,newValue;
	long largeBump;

	largeBump = g->w->portRect.bottom/2;

	oldValue = GetCtlValue(cH);
	switch(part)
		{
		case inUpButton:
			bump = -kSmallBump;
			break;
		case inDownButton:
			bump = kSmallBump;
			break;
		case inPageUp:
			bump = - largeBump;
			break;
		case inPageDown:
			bump = largeBump;
			break;
		default:
			bump = 0;
		}

	SetCtlValue(cH,bump+oldValue);
	newValue = GetCtlValue(cH);
	BumpOrdsTo(newValue);
	}



void ClickDoc(short n,Point p,short mods)
/*
 * Come here for a click in the window.
 */
	{
	Boolean hit;
	OneRegisteredDevice *ord;
	long i;
	long stash;
	short part;
	ControlHandle cH;
	short choice,oldChoice;

	part = FindControl(p,g->w,&cH);
	if(cH == g->scrollBar)
		{
		if(part == inThumb)
			{
			long newValue;

			TrackControl(cH,p,nil);
			newValue = GetCtlValue(cH);
			BumpOrdsTo(newValue);
			}
		else
			TrackControl(cH,p,ScrollAction);
		}


	oldChoice = GetMCPopupChoice(&g->defaultInputPopup);
	hit = ClickMCPopup(&g->defaultInputPopup,p);
	if(hit)
		{
		choice = GetMCPopupChoice(&g->defaultInputPopup);
		if(choice != oldChoice)
			NASetDefaultMIDIInput(g->na,choice == 1 ? nil : &g->sourcePort[choice-1].portConnections);
		goto goHome;
		}





	GoORDClip();
	hit = ClickTEList(&g->teList,p,mods,&stash);
	GoBigClip();
	if(stash >= 0)
		SetOrdName(&g->registeredDevice[stash]);
	if(hit)
		goto goHome;

	UpdateSynthMenu();
	for(i = 1; i <= g->registeredDeviceCount; i++)
		{

		ord = &g->registeredDevice[i-1];
		UpdateDeviceMenu(ord);

		oldChoice = GetMCPopupChoice(&ord->kindPopup);
		hit = ClickMCPopup(&ord->kindPopup,p);
		if(hit)
			{
			choice = GetMCPopupChoice(&ord->kindPopup);
			if(choice != oldChoice)
				{
				SetOrdSynth(ord,choice);
				}
			goto goHome;
			}

		if(ord->hasMIDIBox)
			{
			oldChoice = GetMCPopupChoice(&ord->portPopup);
			hit = ClickMCPopup(&ord->portPopup,p);
			if(hit)
				{
				choice = GetMCPopupChoice(&ord->portPopup);
				if(choice != oldChoice)
					SetOrdPort(ord,choice);
				goto goHome;
				}

			oldChoice = GetMCPopupChoice(&ord->channelPopup);
			hit = ClickMCPopup(&ord->channelPopup,p);
			if(hit)
				{
				choice = GetMCPopupChoice(&ord->channelPopup);
				if(choice != oldChoice)
					SetOrdMIDIChannel(ord,choice);
				goto goHome;
				}
			}

		if(ord->hasDeviceBox)
			{
			oldChoice = GetMCPopupChoice(&ord->devicePopup);
			hit = ClickMCPopup(&ord->devicePopup,p);
			if(hit)
				{
				choice = GetMCPopupChoice(&ord->devicePopup);
				if(choice != oldChoice)
					{
					SetOrdDevice(ord,choice-1);		/* item 1 = "Off" */
					}
				goto goHome;
				}
			}			
		}

goHome:;
	}



void KeyDoc(short n,short key,short code,short mods)
	{
	Boolean took;
	long stash;

	GoORDClip();
	took = KeyTEList(&g->teList,key,mods,&stash);
	if(stash >= 0)
		SetOrdName(&g->registeredDevice[stash]);
	GoBigClip();
	}

void ActivateDoc(short n)
	{
	if(n >= kFirstInfoWindow)
	 	SetMenuItem(mClose,1,0,0,nil);			/* enable "Close" menu item		*/
	}

void DeactivateDoc(short n)
	{
	SetMenuItem(mClose,-1,0,0,nil);				/* disable "Close" menu item	*/
	}

void LetsQuit(short n,short menuItem,short menuRef)
	{
	ComponentResult thisError;

	thisError = NASaveMusicConfiguration(g->na);
	gQuitApp++;
	}


void SaveConfig(short n,short menuItem,short menuRef)
	{
	ComponentResult thisError;

	thisError = NASaveMusicConfiguration(g->na);
	}


void IdleDoc(short n, Boolean front)
	{
	GoORDClip();
	IdleTEList(&g->teList);
	GoBigClip();
	}


void SynthesizerInfo(short n,short menuItem,short menuRef)
	{
	short x;
	OneDevice *io;
	Rect r;

	x = menuRef - mSynthesizerInfo;
	io = &g->device[x];

	if(!io->w)
		{
		r.top = r.left = 0;
		r.bottom = kInfoHeight;
		r.right = kInfoWidth;
		gStaggerWindows = true;
		OffsetRect(&r,10,50);
		io->w = InstallWindow(x + kFirstInfoWindow,io->sd.name,&r,0,wPrintDraw + wCopyDraw,
				DrawInfo,nil,nil,CloseInfo,
				ActivateDoc,DeactivateDoc,nil);
		}
	else
		{
		SelectWindow(io->w);
		}
	}

static void ResizeDoc(short windowNum, Point *oldSize, Point *newSize, short modifiers)
	{
	InvalRect(&gBigRect);
	newSize->h = oldSize->h;
	SizeWindow(g->w,newSize->h,newSize->v,1);
	PositionScrollBar();
	}


void OpenWindow(short n,short menuItem,short menuRef)
	{
	Rect r,r2;
	short i,j,x,y;
	Boolean noGMs;
	OneDevice *od;


	SetRect(&r,0,0,kConfigWindowWidth,200);
	OffsetRect(&r,20,40);

	gStaggerWindows = false;

	g->w = InstallWindow(1,"\pQuickTime Music",&r,0,wPrintDraw + wCopyDraw + wGrowable,
								DrawDoc,ClickDoc,KeyDoc,nil,
								ActivateDoc,DeactivateDoc,IdleDoc);

	SetWindowResizeProc(1,ResizeDoc);

	PositionScrollBar();
	y = kHeaderHeight + kDocMargin;
	BuildRegisteredDeviceList(y);
	PositionScrollBar();



	/*
	 * Make pop up menu input selection
	 */
		{
		Rect r;
	
		r.left = kDefaultInputLeft;
		r.right = kDefaultInputRight;
		r.top = kDefaultInputTop;
		r.bottom = kDefaultInputBottom;

		GoNT();
		InitializeMCPopup(&g->defaultInputPopup,&r,nil,kInputMenuID,1);
		for(i = 0; i < g->sourcePortCount; i++)
			AddMCPopupItem(&g->defaultInputPopup,g->sourcePort[i].portName,i);
		}



	}

void FindSynthesizers(void)
	{
	ComponentDescription cd;
	Component co;
	MusicComponent mc;
	short i,j,dev;
	OneDevice *io,*jo,t;
	ComponentResult result;


	cd.componentType = kMusicComponentType;
	cd.componentSubType = 0;
	cd.componentManufacturer = 0;
	cd.componentFlags = 0;
	cd.componentFlagsMask = 0;

	i = CountComponents(&cd);
	g->deviceCount = 0;
	g->midiDeviceCount = 0;
	co = 0;
	io = &g->device[0];
	while(i--)
		{
		ComponentDescription cd2;

		co = FindNextComponent(co,&cd);
		GetComponentInfo(co,&cd2,nil,nil,nil);
		if((cd2.componentFlags & 1) == 0)			/* flag bit 0 specifies: dont let user see it */
			{
			mc = OpenComponent(co);
			if(mc)
				{
				result = MusicGetDescription(mc,&io->sd);
				if(io->sd.flags & kSynthesizerUsesMIDIPort)
					g->midiDeviceCount ++;
				io->w = nil;
				io->knobCount = MusicGetKnobDescription(mc,0,nil);
				io->instrumentKnobCount = MusicGetInstrumentKnobDescription(mc,0,nil);
				io->drumKnobCount = MusicGetDrumKnobDescription(mc,0,nil);
	
				io->deviceMenu = 0;
				io->hardwareDeviceCount = 0;
	
				if(io->sd.flags & kSynthesizerHardware)
					{
					io->hardwareDeviceCount = MusicGetDeviceConnection(mc,0,
							nil,nil);
					if(io->hardwareDeviceCount < 0)
						io->hardwareDeviceCount = 0;
					if(io->hardwareDeviceCount > 1)
						{
						io->deviceMenuID = g->deviceMenuID++;
						io->deviceMenu = NewOddMenu(io->deviceMenuID);
						AddOddMenuItem(io->deviceMenu,"\pOff",0);
						for(dev = 1; dev <= io->hardwareDeviceCount; dev++)
							{
							Str31 s;
		
							NumToString(dev,s);
							AddOddMenuItem(io->deviceMenu, s,dev);
							}
						}
					else
						io->deviceMenu = 0;
					}
	
				CloseComponent(mc);
				io++;
				g->deviceCount++;
				}
			}
		}

	/*
	 * bubble sort, blow me.
	 */
	io = &g->device[0];
	for(i = 0; i < g->deviceCount; i++)
		{
		jo = io + 1;
		for(j = i + 1; j < g->deviceCount; j++)	
			{
			if(IUMagString(io->sd.name+1,jo->sd.name+1,io->sd.name[0],jo->sd.name[0])
					> 0)
				{
				t = *io;
				*io = *jo;
				*jo = t;
				}
			jo++;
			}
		io++;
		}

	for(i = 0; i < g->deviceCount; i++)
		{
		io = &g->device[i];
		if(io->sd.synthesizerType == kGMType)
			g->defaultDevice = i;
		}
	}

void InitVars()
/*
 * Called once at startup: yes, it
 * inits the vars.
 */
	{
	g = (Globals *)NewPtrClear(sizeof(Globals));

	g->na = OpenDefaultComponent(kNoteAllocatorType,0);
	InitializeTEList(&g->teList);
	g->deviceMenuID = kFirstDeviceMenuID;

		{
		OddMenu om;
		Str255 s;
		short i;

		g->channelMenu = NewOddMenu(kChannelMenuID);
		for(i = 1; i <= 16 ;i++)
			{
			AnyBaseToPString(i,10,0,s);
			AddOddMenuItem(g->channelMenu, s, i);
			}
		}
	}


void FindMIDIPorts(void)
	{
	ComponentResult result;
	Handle inH,outH;
	short i;
	QTMIDIPort *qtmp;
	QTMIDIPort *op;

	result = NAGetMIDIPorts(g->na,&inH,&outH);

	/*
	 * Make list of out-to-synthesizer ports
	 */
	HLock(outH);
	qtmp = (QTMIDIPort *)(*outH + 2);
	i = **(short **)outH;
	
	g->portCount = 1;
	op = &g->port[0];
	ZeroSC(&op->portConnections);
	CopyPString(op->portName,"\pOff");
	op++;

	while(i--)
		{
		*op = *qtmp;
		op++;
		qtmp++;
		g->portCount++;
		}

	DisposeHandle(outH);

	/*
	 * Make list of input-from-keyboard ports
	 */
	HLock(inH);
	qtmp = (QTMIDIPort *)(*inH + 2);
	i = **(short **)inH;
	
	g->sourcePortCount = 1;
	op = &g->sourcePort[0];
	ZeroSC(&op->portConnections);
	CopyPString(op->portName,"\pOff");
	op++;

	while(i--)
		{
		*op = *qtmp;
		op++;
		qtmp++;
		g->sourcePortCount++;
		}

	DisposeHandle(inH);

	/*
	 * Make pop up menu for midi port selection
	 */
	g->portMenu = NewOddMenu(kPortMenuID);
	for(i = 0; i < g->portCount; i++)
		AddOddMenuItem(g->portMenu, g->port[i].portName,i+1);
	}



void UpdateDeviceMenu(OneRegisteredDevice *ord)
	{
	OneDevice *od;
	OneRegisteredDevice *ord2;
	short i,j;
	short lowestOpen;

	WhichSynth(ord->synthType,&od);
	if(!od->deviceMenu)
		goto goHome;

	for(i = 0; i <= od->hardwareDeviceCount; i++)
		{
		EnableOddMenuItem(od->deviceMenu,i,true);
		for(j = 0; j < g->registeredDeviceCount; j++)
			{
			ord2 = &g->registeredDevice[j];
			if(i && ord2->synthType == ord->synthType && ord2->sc.midiChannel == i)
				EnableOddMenuItem(od->deviceMenu,i,false);
			}
		} 
goHome:;
	}


void UpdateSynthMenu(void)
	{
	long i,j;
	OneDevice *od;
	OneRegisteredDevice *ord;
	short result;
	short hardwareUsedCount;

	result = 0;

	for(i = 0; i < g->deviceCount; i++)
		{
		od = &g->device[i];
		hardwareUsedCount = 0;
		EnableOddMenuItem(g->synthMenu,i+1,true);

		for(j = 0; j < g->registeredDeviceCount; j++)
			{
			ord = &g->registeredDevice[j];
			if(od->sd.flags & kSynthesizerHardware)
			/*
			 * Disable hardware synth menu item after all are used up
			 */
				{
				if(od->sd.synthesizerType == ord->synthType)
					hardwareUsedCount++;

				if(hardwareUsedCount >= od->hardwareDeviceCount)
					EnableOddMenuItem(g->synthMenu,i+1,false);
				}
			else if(!(od->sd.flags & kSynthesizerUsesMIDIPort))
			/*
			 * Disable software synth menu item if even one is used
			 */
				{
				if(od->sd.synthesizerType == ord->synthType)
					EnableOddMenuItem(g->synthMenu,i+1,false);
				}
			}
		}

goHome:;
	}

short WhichSynth(OSType synthType,OneDevice **odOut)
	{
	short i;
	OneDevice *od;
	short result;

	result = 0;

	for(i = 0; i < g->deviceCount; i++)
		{
		od = &g->device[i];
		if(synthType == od->sd.synthesizerType)
			{
			result = i + 1;
			goto goHome;
			}
		}
goHome:
	if(odOut)
		*odOut = od;

	return result;
	}

short WhichOrdSynth(OneRegisteredDevice *ord)
	{
	return WhichSynth(ord->synthType,nil);
	}


short WhichPort(OneRegisteredDevice *ord)
	{
	short i;
	QTMIDIPort *op;
	short result;

	result = 1;

	for(i = 0; i < g->portCount; i++)
		{
		op = &g->port[i];
		if(EqualSC(&ord->sc,&op->portConnections) )
			{
			result = i + 1;
			goto goHome;
			}
		}

goHome:
	return result;		
	}

void FigureHases(OneRegisteredDevice *ord)
	{
	OneDevice *od;
	short deviceIndex;

	deviceIndex = WhichOrdSynth(ord);
	od = &g->device[deviceIndex-1];

	ord->hasDeviceBox = ((od->sd.flags & kSynthesizerHardware)
			&& (od->hardwareDeviceCount > 1)) != 0;

	ord->hasMIDIBox = (od->sd.flags & kSynthesizerUsesMIDIPort) != 0;
	}


void UnformatOneRegisteredDevice(OneRegisteredDevice *ord)
/*
 * Close enough stuff that FormatOne will work correctly
 */
	{
	if(ord->synthNameTEID)
		{
		GoORDClip();
		RemoveTEField(&g->teList,ord->synthNameTEID);
		ord->synthNameTEID = 0;
		GoBigClip();
		}
	}


short FormatOneRegisteredDevice(OneRegisteredDevice *ord,short y)
	{
	Rect r;
	short choice;
	OneDevice *od;

	FigureHases(ord);

	ord->ordTop = y;
	ord->ordBottom = ord->ordTop + kOrdRowHeight;

	ord->ordTop;

	r.left = kOrdSynthNameLeft;
	r.right = kOrdSynthNameRight;
	r.top = ord->ordTop + (kOrdRowHeight - kTEListBox9ptHeight)/2;
	r.bottom = r.top + kTEListBox9ptHeight;

	GoBT();
	ord->synthNameTEID = AddTEField(&g->teList,&r,ord->synthName,31);

	r.left = kOrdSynthTypeLeft + kMCPopupBoxMargin;
	r.right = kOrdSynthTypeRight - kMCPopupBoxMargin;
	r.top = ord->ordTop + (kOrdRowHeight - kPopupHeight)/2;
	r.bottom = r.top + kPopupHeight;

	choice = WhichSynth(ord->synthType,&od);

	GoBT();
	InitializeMCPopup(&ord->kindPopup,&r,g->synthMenu,kSynthMenuID,choice);

	if(ord->hasMIDIBox)
		{
		r.left = kOrdSynthMIDIPortLeft + kMCPopupBoxMargin;
		r.right = kOrdSynthMIDIPortRight - kMCPopupBoxMargin;
		choice = WhichPort(ord);
		InitializeMCPopup(&ord->portPopup,&r,g->portMenu,kPortMenuID,choice);

		r.left = kOrdSynthChannelLeft + kMCPopupBoxMargin;
		r.right = kOrdSynthChannelRight - kMCPopupBoxMargin;
		InitializeMCPopup(&ord->channelPopup,&r,g->channelMenu,kChannelMenuID,ord->sc.midiChannel);
		}

	if(ord->hasDeviceBox)
		{
		r.left = kOrdSynthMIDIPortLeft + kMCPopupBoxMargin;
		r.right = kOrdSynthMIDIPortRight - kMCPopupBoxMargin;
		choice = ord->sc.midiChannel;
		InitializeMCPopup(&ord->devicePopup,&r,od->deviceMenu,od->deviceMenuID,choice);
		}

	return ord->ordBottom;
	}


void BuildRegisteredDeviceList(long y)
	{
	short i;
	OneRegisteredDevice *ord;

	g->registeredDeviceCount = NAGetRegisteredMusicDevice(g->na,0,
			nil,nil,nil,nil);

	for(i = 1; i <= g->registeredDeviceCount; i++)
		{
		ord = &g->registeredDevice[i-1];

		NAGetRegisteredMusicDevice(g->na,i,
			&ord->synthType,ord->synthName,&ord->sc,&ord->mc);

		y = FormatOneRegisteredDevice(ord,y);
		}
	}


void Bootstrap()
	{
	short i;
	Str255 s;

	EnterMovies();
	InitVars();

/*** File Menu ***/
	InstallMenu("\pFile",nil,0);

	InstallMenuItem("\pClose/W",(void *)CloseInfo,-mClose);
	InstallMenuItem("\p(-",nil,0);
	InstallMenuItem("\pSave Configuration/S",SaveConfig,mSave);
	InstallMenuItem("\p(-",nil,0);
	InstallPrintItems(nil,nil);
	InstallMenuItem("\p(-",nil,0);
	InstallQuitItem(LetsQuit,0);

/*** Edit Menu ***/
	InstallEditMenu(nil,nil,nil,nil,nil);
	InstallMenuItem("\p(-",nil,0);
	InstallMenuItem("\pAdd Synthesizer/A",AddSynth,mAddSynth);
	InstallMenuItem("\pRemove Synthesizer/R",RemoveSynth,mRemoveSynth);

	FindSynthesizers();

/*** Synthesizer Menu ***/
	InstallMenu("\pSynthesizer Info",nil,0);

	g->synthMenu = NewOddMenu(kSynthMenuID);

	for(i = 0; i < g->deviceCount; i++)
		{
		AddOddMenuItem(g->synthMenu, g->device[i].sd.name,i+1);

		CopyPString(s,g->device[i].sd.name);
		s[++s[0]] = '';
		if(i < 9)
			{
			s[++s[0]] = '/';
			s[++s[0]] = i + '1';
			}
		InstallMenuItem(s,SynthesizerInfo,mSynthesizerInfo+i);
		}
		

	FindMIDIPorts();
	OpenWindow(0,0,0);
	}

void Hatstrap()
/*
  * clean up
  */
	{
	}


short GetModKeys(void)
/*
 * return the modifier keys just like in an eventrecord
 */
	{
	short result;
	KeyMap keys;

	GetKeys(keys);

	result = 0;
	if(keys[1] & 1)
		result |= shiftKey;
	if(keys[1] & 4)
		result |= optionKey;
	if(keys[1] & 8)
		result |= controlKey;
	if(keys[1] & 0x8000)
		result |= cmdKey;

	return result;
	}


void ZeroSC(SynthesizerConnections *sc)
	{
	sc->clientID = 0;
	sc->inputPortID = 0;
	sc->outputPortID = 0;
	sc->midiChannel = 1;
	sc->flags = 0;
	sc->unique = 0;
	sc->reserved1 = 0;
	sc->reserved2 = 0;
	}

Boolean EqualSC(SynthesizerConnections *sc1,SynthesizerConnections *sc2)
	{
	return sc1->clientID == sc2->clientID
			&& sc1->inputPortID == sc2->inputPortID
			&& sc1->outputPortID == sc2->outputPortID
			&& sc1->flags == sc2->flags;
	}