/*
	File:		QTMusicConfiguration.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	

	Change History (most recent first):

		 <6>	11/14/95	dvb		Conditionally, use public interfaces.
		 <5>	10/19/95	dvb		Get rid of ridiculous 16-checkbox midi channel selector.
		 <4>	 6/10/95	dvb		Latency in the Info window
		 <3>	 28-4-95	dvb		input selection
		 <2>	  8-3-95	dvb		Deal correctly with oms/midimgr ports

*/


#include "NAEditInstSlider.h"
#include "TELists.h"

#ifdef UsePublicMusicInterfaces
	#include <QuickTimeComponents.h>
#else
	#include "NoteAllocatorComponent.h"
#endif


/*--------------------------
	Limits and Konstants
--------------------------*/

#define kMaxMusicDevices 128
#define kMaxPorts 16
#define kMaxRegisteredDevices 128
#define kInputNumber 100



typedef struct
	{
	short port;		/* index of which port, 0 for none, OR 1 for on, 0 for off, if soft */
	SynthesizerDescription sd;
	short knobCount;
	short instrumentKnobCount;
	short drumKnobCount;

	short hardwareDeviceCount;		/* -1 if the device count is unknown */
	long deviceUsed;				/* array of boolean, bit 0 is first device, 1<<(n - 1) */
	long deviceMenuID;
	OddMenu deviceMenu;			/* array of numbers 1 to hardwareDeviceCount */

	WindowPtr w;	/* info window for device */
	} OneDevice;

typedef struct
	{
	SynthesizerConnections sc;
	OSType synthType;
	Str31 synthName;

	MusicComponent mc;
	SynthesizerDescription sd;

	long ordTop,ordBottom;				/* entire box */


	Boolean hasUnitBox;				/* box for selecting which hardware device */
	Boolean hasMIDIBox;				/* box for selecting midi port channels used */
	Boolean hasDeviceBox;			/* box for selecting which piece of hardware to use */

	long synthNameTEID;
	MCPopup kindPopup;

	MCPopup portPopup;
	MCPopup channelPopup;
//	Chex chex;					/* midi checkboxes */

	MCPopup devicePopup;

	} OneRegisteredDevice;



#define kSynthMenuID 4014
#define kPortMenuID 4020
#define kInputMenuID 7414
#define kChannelMenuID 6502
#define kFirstDeviceMenuID 4040

typedef struct
	{
	short deviceCount;
	short midiDeviceCount;					/* how many require MIDI connections? */
	OneDevice device[kMaxMusicDevices];

	short defaultDevice;					/* add synth uses it */

	short portCount;						/* how many MIDI In ports we find, port 0 == "off" */
	QTMIDIPort port[kMaxPorts];
	OneDevice inDevice;

	short sourcePortCount;
	QTMIDIPort sourcePort[kMaxPorts];
	MCPopup defaultInputPopup;

	short registeredDeviceCount;
	OneRegisteredDevice registeredDevice[kMaxRegisteredDevices];

	OddMenu synthMenu;
	OddMenu portMenu;
	OddMenu channelMenu;

	WindowPtr w;
	NoteAllocator na;

	TEList teList;
	Rect scrollBarR;
	ControlHandle scrollBar;
	long currentBump;

	long deviceMenuID;
	} Globals;

#ifdef _Globals_
	Globals *g;
#else
	extern Globals *g;
#endif

#define kFirstInfoWindow 3

#define kHeaderTop 30
#define kHeaderHeight (kHeaderTop+20)


//----------------------------------------------

#define kDefaultInputLeft 110
#define kDefaultInputRight  (kOrdSynthMIDIPortWidth + kDefaultInputLeft)
#define kDefaultInputTop 10
#define kDefaultInputBottom (kPopupHeight+kDefaultInputTop)

#define kInfoColumn 135
#define kFirstInfoRow 20
#define kInfoRowHeight 15
#define kInfoHeight (kFirstInfoRow + (kInfoRowHeight)*12)
#define kInfoWidth 250

#define kConfigWidth (kOrdSynthChannelRight+kDocMargin)
#define kConfigWindowWidth (kConfigWidth+15)

#define kDocMargin 10



#define kNamedRectMargin 12
#define kNamedRectNameGapLeft 7
#define kNamedRectNameGapRight 5
#define kNamedRectNameHeight 11
#define kNamedRectNameBase 3

#define kNamedRectInset (2*kNamedRectMargin)

#define k9PtTextHeight 10



/*
 * Formatting for MIDI box
 */

//#define kBoxFieldNameX 110
#define kBoxFieldETMargin 3
#define kBoxFieldETHeight 18
#define kBoxFieldETBump 2






#define kPopupHeight 10


#define kOrdRowHeight 20
#define kOrdColumnGap 12

#define kOrdSynthNameWidth 100
#define kOrdSynthTypeWidth 100
#define kOrdSynthMIDIPortWidth 100
#define kOrdSynthChannelWidth 40

#define kOrdSynthNameLeft kDocMargin
#define kOrdSynthNameRight (kOrdSynthNameLeft + kOrdSynthNameWidth)

#define kOrdSynthTypeLeft (kOrdSynthNameRight + kOrdColumnGap)
#define kOrdSynthTypeRight (kOrdSynthTypeLeft+kOrdSynthTypeWidth)

#define kOrdSynthMIDIPortLeft (kOrdSynthTypeRight + kOrdColumnGap)
#define kOrdSynthMIDIPortRight (kOrdSynthMIDIPortLeft + kOrdSynthNameWidth)

#define kOrdSynthChannelLeft (kOrdSynthMIDIPortRight + kOrdColumnGap)
#define kOrdSynthChannelRight (kOrdSynthChannelLeft + kOrdSynthChannelWidth)


/*--------------------------
	Prototypes
--------------------------*/
void DrawDoc(short n);
void DrawInfo(short n);
void CloseInfo(short n);
void ClickDoc(short n,Point p,short mods);
void KeyDoc(short n,short key,short code,short mods);
void ActivateDoc(short n);
void DeactivateDoc(short n);
void LetsQuit(short n,short menuItem,short menuRef);
void SaveConfig(short n,short menuItem,short menuRef);
void OpenWindow(short n,short menuItem,short menuRef);
void InitVars(void);
void SynthesizerInfo(short n,short menuItem,short menuRef);

void BuildRegisteredDeviceList(long y);

short GetModKeys(void);

void FindSynthesizers(void);
void FindMIDIPorts(void);

void DrawHeader(void);


void GoNT(void);
void GoBT(void);
void GoORDClip(void);
void GoBigClip(void);

