/*
	File:		QTMusicConfigurationDrawing.c

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	

	Change History (most recent first):

		 <6>	11/14/95	dvb		Conditionally, use public interfaces.
		 <5>	10/19/95	dvb		Get rid of ridiculous 16-checkbox midi channel selector.
		 <4>	 6/14/95	JB		added prototypes
		 <3>	 6/10/95	dvb		Latency in the Info window
		 <2>	 28-4-95	dvb		input selection

*/

#include "QTMusicConfiguration.h"

#include <QuickDraw.h>
#include "BigEasyGrafish.h"
#include "BigEasyTextish.h"

void GoB(void);
void GoNT(void);
void GoBT(void);
void DrawSynthesizerBox(OneRegisteredDevice *ord);
void DrawMIDIBox(OneRegisteredDevice *ord);
void DrawDeviceBox(OneRegisteredDevice *ord);
void DrawDivideLine(short y, Boolean wide, Boolean black);
void DrawOneRegisteredDevice(OneRegisteredDevice *ord);
void DrawInputSelect(void);
void DrawHeader(void);
void GoORDClip(void);
void GoBigClip(void);
void DrawDoc(short n);
void DrawInfo(short n);

void GoB(void)
	{
	ForeColor(blackColor);
	PenNormal();
	}

void GoNT(void)
	{
	GoB();
	TextFont(1);
	TextSize(9);
	TextFace(0);
	}

void GoBT(void)
	{
	GoB();
	TextFont(1);
	TextSize(9);
	TextFace(bold);
	}


void DrawSynthesizerBox(OneRegisteredDevice *ord)
	{
	DrawTEField(&g->teList,ord->synthNameTEID);
	DrawMCPopup(&ord->kindPopup);
	}

void DrawMIDIBox(OneRegisteredDevice *ord)
	{
	if(ord->hasMIDIBox)
		{
		DrawMCPopup(&ord->portPopup);
		DrawMCPopup(&ord->channelPopup);
		}
	else
		{
		Rect r;

		r.left = kOrdSynthMIDIPortLeft;
		r.right = kOrdSynthChannelRight;
		r.top = ord->ordTop;
		r.bottom = ord->ordBottom;
		EraseRect(&r);
		}
	}

void DrawDeviceBox(OneRegisteredDevice *ord)
	{
	if(ord->hasDeviceBox)
		{
		DrawMCPopup(&ord->devicePopup);
		}
	else
		{
		Rect r;

		r.left = kOrdSynthMIDIPortLeft;
		r.right = kOrdSynthChannelRight;
		r.top = ord->ordTop;
		r.bottom = ord->ordBottom;
		EraseRect(&r);
		}
	}


void DrawDivideLine(short y,Boolean wide,Boolean black)
/*
 * Draw double line above the y-point.
 */
	{
	short r;

	r = wide ? kConfigWindowWidth : kConfigWidth;
	GoB();
	MoveTo(0,y-1);
	Line(r,0);
	MoveTo(0,y-3);
	Line(r,0);
	if(!black)
		GoWhite();
	MoveTo(0,y-2);
	Line(r,0);
	GoBlack();
	}


void DrawOneRegisteredDevice(OneRegisteredDevice *ord)
	{
	GoBW();
	GoORDClip();

	DrawSynthesizerBox(ord);
	DrawMIDIBox(ord);
	if(ord->hasDeviceBox)
		DrawDeviceBox(ord);

	GoBigClip();
	}

void DrawInputSelect(void)
	{
	Rect r;
	short y;

	GoNT();

	r = qd.thePort->portRect;
	r.bottom = kHeaderTop;
	EraseRect(&r);
	MoveTo(r.left,r.bottom-1);
	LineTo(r.right,r.bottom-1);

	MoveTo(kDefaultInputLeft,kDefaultInputBottom);
	DrawStringRight("\pDefault MIDI Input: ");
	DrawMCPopup(&g->defaultInputPopup);
	}	

void DrawHeader(void)
	{
	Rect r;
	short y;

	GoNT();

	r = qd.thePort->portRect;
	r.top = kHeaderTop;
	r.bottom = kHeaderHeight;
	EraseRect(&r);

	y = kHeaderHeight - 3;
	MoveTo(r.left,y);
	LineTo(r.right,y);
	y = kHeaderHeight - 1;
	MoveTo(r.left,y);
	LineTo(r.right,y);

	y = kHeaderHeight - 7;

	MoveTo(kOrdSynthNameLeft,y);
	DrawString("\pName");

	MoveTo(kOrdSynthNameLeft,y);
	DrawString("\pName");

	MoveTo(kOrdSynthTypeLeft,y);
	DrawString("\pSynthesizer");

	MoveTo(kOrdSynthMIDIPortLeft,y);
	DrawString("\pMIDI Port/Card Number");

	MoveTo(kOrdSynthChannelLeft,y);
	DrawString("\pMIDI Channel");
	}

void GoORDClip(void)
	{
	Rect r;

	r.top = kHeaderHeight;
	r.bottom = 10000;
	r.left = 0;
	r.right = kConfigWindowWidth;
	ClipRect(&r);
	}

void GoBigClip(void)
	{
	Rect r;

	r.left = r.top = -30000;
	r.bottom = r.right = 30000;
	ClipRect(&r);
	}

void DrawDoc(short n)
	{
	short i;
	Rect r;

	EraseRect(&qd.thePort->portRect);

	DrawInputSelect();
	DrawHeader();

	r.left = 0;
	r.right = kConfigWindowWidth;
	r.top = kHeaderHeight;
	r.bottom = kHeaderHeight-1;
	EraseRect(&r);

	for(i = 0; i < g->registeredDeviceCount; i++)
		DrawOneRegisteredDevice(&g->registeredDevice[i]);

	GoBT();
	GoORDClip();
//	DrawTEList(&g->teList);
	GoBigClip();
	}



void DrawInfo(short n)
	{
	OneDevice *io;
	long x;
	long row;

	n -= kFirstInfoWindow;
	io = &g->device[n];

	EraseRect(&qd.thePort->portRect);

	GoNT();

	MoveTo(kInfoColumn,kFirstInfoRow);
	DrawString(io->sd.name);

	row = 2;
	//  Kind
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pKind: ");
	TextFace(0);
	if(io->sd.flags & kSynthesizerUsesMIDIPort)
		DrawString("\pMIDI");
	else if(io->sd.flags & kSynthesizerSoftware)
		DrawString("\pSoftware");
	else if(io->sd.flags & kSynthesizerHardware)
		DrawString("\pHardware");
	else
		DrawString("\pUnknown");
	DrawString("\p Synthesizer");

	row++;

	//  Polyphony
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pPolyphony: ");
	TextFace(0);
	DrawNum(io->sd.voiceCount);

	row++;

	//  Model
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pModel: ");
	TextFace(0);
	if(io->sd.flags & kSynthesizerGMSuperset)
		DrawString("\pGeneral MIDI Superset");
	else if(io->sd.flags & kSynthesizerGM)
		DrawString("\pGeneral MIDI");
	else
		DrawString("\pSpecific");

	row++;

	//  latency
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pLatency: ");
	TextFace(0);
	DrawNum(io->sd.latency);

	row++;

	//  Built-In count
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pBuilt-in Instruments: ");
	TextFace(0);
	if(io->sd.flags & kSynthesizerGMSuperset)
		DrawString("\pGM + ");
	DrawNum(io->sd.instrumentCount);
	DrawString("\p / ");
	if(io->sd.flags & kSynthesizerGMSuperset)
		DrawString("\pGM + ");
	DrawNum(io->sd.drumCount);


	row++;

	//  User Inst count
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pUser Instruments: ");
	TextFace(0);
	DrawNum(io->sd.modifiableInstrumentCount);
	DrawString("\p / ");
	DrawNum(io->sd.modifiableDrumCount);


	row++;

	//  Parts
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pParts: ");
	TextFace(0);
	if(io->sd.flags & kSynthesizerMixedDrums)
		x = io->sd.partCount;
	else
		x = io->sd.partCount + io->sd.drumPartCount;
	DrawNum(x);


	row++;

	//  Global Knobs
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pGlobal Knobs: ");
	TextFace(0);
	DrawNum(io->knobCount);


	row++;

	//  Instrument Knobs
	MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
	TextFace(bold);
	DrawStringRight("\pInstrument Knobs: ");
	TextFace(0);
	DrawNum(io->instrumentKnobCount);
	DrawString("\p / ");
	DrawNum(io->drumKnobCount);


	row++;

	//  Device Count
	if(io->sd.flags & kSynthesizerHardware)
		{
		MoveTo(kInfoColumn,kFirstInfoRow + kInfoRowHeight * row);
		TextFace(bold);
		DrawStringRight("\pUnits Present: ");
		TextFace(0);
		DrawNum(io->hardwareDeviceCount);
		}


	}


