/*
	File:		TELists.c

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 6/14/95	JB		added prototypes

*/


#include <Memory.h>
#include <Events.h>
#include "TELists.h"

void TEDisposeAt(TEHandle *teH);
long FindTEField(TEList *teList, long id, OneTEField **otefOut);
void DrawTEListField(TEList *teList, short i);
void UpdateFieldText(TEList *teList);
void ActivateField(TEList *teList, short i, Boolean selectAll);


void TEDisposeAt(TEHandle *teH)
	{
	if(*teH)
		{
		TEDispose(*teH);
		*teH = 0;
		}
	}



void InitializeTEList(TEList *teList)
	{
	GrafPtr thePort;

	teList->seed = 1;
	teList->teFieldCount = 0;
	teList->teH = nil;
	teList->active = -1;

	GetPort(&thePort);
	}


void TerminateTEList(TEList *teList)
	{
	TEDisposeAt(&teList->teH);
	}



long AddTEField(TEList *teList,Rect *r,StringPtr text,long maxLength)
	{
	OneTEField *otef;
	GrafPtr thePort;

	otef = teList->teField + teList->teFieldCount;
	teList->teFieldCount++;

	GetPort(&thePort);
	otef->font = thePort->txFont;
	otef->size = thePort->txSize;
	otef->face = thePort->txFace;

	otef->seed = teList->seed++;
	otef->r = *r;
	otef->frameR = *r;
	InsetRect(&otef->frameR,-kTEListBoxMargin,-kTEListBoxMargin);
	otef->text = text;
	otef->maxLength = maxLength;

	return otef->seed;
	}

long FindTEField(TEList *teList,long id,OneTEField **otefOut)
	{
	long i;
	OneTEField *otef;

	for(i = 0; i < teList->teFieldCount; i++)
		{
		otef = teList->teField + i;
		if(otef->seed == id)
			goto goHome;
		}
	i = -1;
	otef = 0;
goHome:
	if(otefOut)
		*otefOut = otef;
	return i;
	}

void RemoveTEField(TEList *teList,long id)
	{
	long i;
	OneTEField *otef;

	i = FindTEField(teList,id,&otef);

	if(otef)
		{
		if(i == teList->active)				/* dont delete an active field */
			ActivateField(teList,-1,false);
		if(teList->teFieldCount > 1)
			*otef = teList->teField[teList->teFieldCount-1];
		teList->teFieldCount--;
		goto goHome;
		}
goHome:;
	}


void DrawTEListField(TEList *teList,short i)
	{
	OneTEField *otef;
	otef = teList->teField + i;

	TextFont(otef->font);
	TextSize(otef->size);
	TextFace(otef->face);

	FrameRect(&otef->frameR);
	if(teList->active == i && teList->teH)
		TEUpdate(&otef->r, teList->teH);
	else
		TETextBox(otef->text+1,otef->text[0],&otef->r,0);
	}

void DrawTEList(TEList *teList)
	{
	long i;
	Rect r;

	for(i = 0; i < teList->teFieldCount; i++)
		DrawTEListField(teList,i);
	}


void DrawTEField(TEList *teList,long id)
	{
	long i;

	i = FindTEField(teList,id,nil);
	if(i >= 0)
		DrawTEListField(teList,i);
	}


void UpdateFieldText(TEList *teList)
	{
	OneTEField *otef;
	StringHandle textH;
	long textLength;

	otef = teList->teField + teList->active;

	textH = (StringHandle)(**teList->teH).hText;
	textLength = (**teList->teH).teLength;
	if(textLength <= otef->maxLength)
		{
		otef->text[0] = textLength;
		BlockMove(*textH,otef->text+1,textLength);
		}
	else
		{
		SysBeep(1);
		textLength = otef->maxLength;
		otef->text[0] = textLength;
		BlockMove(*textH,otef->text+1,textLength);

		TESetText(otef->text+1, otef->text[0], teList->teH);
		TEUpdate(&otef->r, teList->teH);
		}
	}

void IdleTEList(TEList *teList)
	{
	if(teList->teH)
		TEIdle(teList->teH);
	}

void ActivateField(TEList *teList,short i,Boolean selectAll)
	{
	OneTEField *otef;
	long oldActive;

	otef = teList->teField + i;
	oldActive = teList->active;

	if((teList->teFieldCount == 0) || (i == oldActive))
		goto goHome;

	teList->active = i;

	if(oldActive >= 0)
		{
		TEDisposeAt(&teList->teH);
		DrawTEListField(teList,oldActive);
		}

	if(teList->active >= 0)
		{
		Rect wide;

		wide = otef->r;
		wide.right = 32000;

		TextFont(otef->font);
		TextSize(otef->size);
		TextFace(otef->face);

		teList->teH = TENew(&wide,&otef->r);
		TEActivate(teList->teH);
		TEAutoView(true,teList->teH);
		TESetText(otef->text+1, otef->text[0], teList->teH);
		if(selectAll)
			TESetSelect(0, 1000, teList->teH);
		teList->changed = false;
		}

goHome:;
	}

Boolean ClickTEList(TEList *teList,Point p,short mods,long *stashIt)
/*
 * return true if the click was for us.
 * set stashIt to index of field that should be made permanent
 */
	{
	OneTEField *otef;
	Boolean shift;
	long i;
	long oldActive,newActive;
	Boolean hit;
	long stash;

	shift = (mods & shiftKey) != 0;
	oldActive = teList->active;

	hit = false;

	for(i = 0; i < teList->teFieldCount; i++)
		{
		otef = teList->teField + i;

		if(PtInRect(p,&otef->frameR))
			{
			ActivateField(teList,i,false);
			TEClick(p, shift, teList->teH);
			hit = true;
			goto doneClick;
			}
		}

	ActivateField(teList,-1,false);

doneClick:
	if((teList->active != oldActive) && teList->changed)
		stash = oldActive;
	else
		stash = -1;
	if(stashIt)
		*stashIt = stash;

	return hit;
	}


Boolean KeyTEList(TEList *teList,short key,short mods,long *stashIt)
/*
 * return true if we took the key,
 * set stashIt an index 0 - (n-1) if the user returns or tabs (locks this setting) away.
 */
	{
	Boolean took;
	long stash;

	took = true;
	stash = -1;

	switch(key)
		{
		case 9:		/* tab */
			stash = teList->changed ? teList->active : -1;
			if(teList->teFieldCount)
				{
				short x;

				x = teList->active + (mods & shiftKey ? -1 : 1);
				if(x < 0)
					x = teList->teFieldCount - 1;
				else if(x >= teList->teFieldCount)
					x = 0;

				ActivateField(teList,x,true);
				}
			break;

		case 13:	/* cr */
			stash = teList->changed ? teList->active : -1;
			break;

		default:
			if(teList->teH)
				{
				TEKey(key,teList->teH);
				teList->changed = true;
				UpdateFieldText(teList);
				}
			break;

		}

	if(stashIt)
		*stashIt = stash;
	return took;
	}

void ActivateTEField(TEList *teList,long id,Boolean selectAll)
	{
	long i;

	i = FindTEField(teList,id,nil);
	ActivateField(teList,i,selectAll);
	}

void SetTEField(TEList *teList,long id,StringPtr s)
	{
	}


void BumpTEField(TEList *teList,long id,short bump)
	{
	long i;
	OneTEField *otef;

	i = FindTEField(teList,id,&otef);

	if(otef)
		{
		if(i == teList->active)
			{
			ActivateField(teList,-1,false);
			}

		OffsetRect(&otef->r,0,bump);
		OffsetRect(&otef->frameR,0,bump);
		}
	}


extern void MyNumToString(long n,short g,StringPtr c);
